﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Text.RegularExpressions;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 棋譜
    /// </summary>
    public class Kifu
    {

        #region プロパティー類
        //------------------------------------------------------------
        private List<MasuBango> masuBangoList;

        /// <summary>
        /// 符号リスト
        /// </summary>
        public List<MasuBango> MasuBangoList
        {
            get
            {
                return this.masuBangoList;
            }
        }
        //------------------------------------------------------------
        #endregion




        public Kifu()
        {
            this.masuBangoList = new List<MasuBango>();
        }

        public Kifu Clone()
        {
            Kifu kifu2 = new Kifu();

            foreach (MasuBango masuBangoFugo in this.MasuBangoList)
            {
                kifu2.MasuBangoList.Add(masuBangoFugo);
            }

            return kifu2;
        }

        /// <summary>
        /// 手目。
        /// </summary>
        public int LastTeme
        {
            get
            {
                return this.MasuBangoList.Count;
            }
        }

        private int LastIndex
        {
            get
            {
                return this.MasuBangoList.Count - 1;
            }
        }

        public MasuBango Last
        {
            get
            {
                MasuBango last = null;

                if (0 != this.MasuBangoList.Count)
                {
                    last = this.MasuBangoList.Last();
                }

                return last;
            }
        }

        /// <summary>
        /// 棋譜に符号を追加。
        /// 
        /// ①コマ送り用。
        /// ②「成り」フラグの更新用。
        /// ③マウス操作用
        /// 
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="masuTime"></param>
        /// <param name="dou"></param>
        /// <param name="komaSyurui"></param>
        /// <param name="migiHidari"></param>
        /// <param name="agaruHiku"></param>
        /// <param name="nari"></param>
        /// <param name="daHyoji"></param>
        public void AddAndRefresh(
            MasuBango masu,
            out MasuBango newMasuBangoFugo
            )
        {
            newMasuBangoFugo = new MasuBango(
                    masu.Sengo,
                    masu.Okiba,
                    masu.SrcOkiba,
                    masu.Suji,
                    masu.Dan,
                    masu.SrcSuji,
                    masu.SrcDan,
                    masu.Syurui,
                    masu.SrcSyurui
                );

            this.masuBangoList.Add(newMasuBangoFugo);
        }


        /// <summary>
        /// 最初の要素を切り取って返します。なければヌル。
        /// </summary>
        /// <returns></returns>
        public void PopFirst(out MasuBango first)
        {
            first = null;

            if(0<this.MasuBangoList.Count)
            {
                first = this.MasuBangoList[0];

                this.MasuBangoList.RemoveAt(0);
            }
        }


        /// <summary>
        /// 最後の要素を切り取って返します。なければヌル。
        /// </summary>
        /// <returns></returns>
        public void PopLast(out MasuBango last)
        {
            last = this.Last;

            if (null != last)
            {
                this.MasuBangoList.RemoveAt(this.MasuBangoList.Count - 1);
            }
        }

        public void ElementAt(int index, out MasuBango masuBangoFugo)
        {
            masuBangoFugo = null;

            if(0<=index && index<this.MasuBangoList.Count)
            {
                masuBangoFugo = this.MasuBangoList[index];
            }
        }


        public void Clear()
        {
            this.MasuBangoList.Clear();
        }

    }


}
