﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Xenon.KifuNarabe
{
    public abstract class Translator_MasuToFugo1
    {
        /// <summary>
        /// 符号１「▲７六歩」等を作ります。
        /// 
        /// 「移動した駒の種類」
        /// 「移動先の升番号」
        /// 「局面」
        /// 「棋譜」
        /// 
        /// があれば作れます。
        /// 
        /// ＜マウス操作で入力したときに呼び出されます＞
        /// </summary>
        /// <returns></returns>
        public static View_Fugo TranslateMasuToFugo1(

            MasuBango process,//移動先、移動元、両方のマス番号

            KomaCollection komas,
            Kifu kifu
            )
        {
            MigiHidari migiHidari;
            AgaruHiku agaruHiku;
            NariFunari nari;
            DaHyoji daHyoji = DaHyoji.Invisible; // “打”表示は、駒を打ったときとは異なります。


            switch (process.SrcSyurui)//「▲２二角成」なら、馬（dst）ではなくて角（src）。
            {
                #region 歩
                //************************************************************
                // 歩
                //************************************************************
                case KomaSyurui.FU:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //┌─┬─┬─┐
                        //│  │  │  │
                        //├─┼─┼─┤
                        //│  │至│  │
                        //├─┼─┼─┤
                        //│  │Ｅ│  │
                        //└─┴─┴─┘
                        MasuBango src = process.Src();

                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(KomanoKidou.SrcE_Oh(process.Sengo, process)))
                        {
                            // Ｅにいた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                        }
                        else
                        {
                            // どこからか飛んできた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                        }

                        //----------
                        // 右左、引上はありません。
                        //----------
                        migiHidari = MigiHidari.None;
                        agaruHiku = AgaruHiku.None;

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.None;
                        }
                    }
                    break;
                #endregion

                #region 飛
                //************************************************************
                // 飛
                //************************************************************
                case KomaSyurui.HISYA:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │  │  │  │  │  │  │  │  │A7│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │A3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A0│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │G7│G6│G5│G4│G3│G2│G1│G0│至│C0│C1│C2│C3│C4│C5│C6│C7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E0│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │E4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E7│  │  │  │  │  │  │  │  │
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        MasuBango src = process.Src();

                        //----------
                        // 棋譜の現局面：競合駒
                        //----------
                        Button_Koma[] kmA = komas.Kyokumen.KmA_Hisya(process.Sengo, process);
                        Button_Koma[] kmC = komas.Kyokumen.KmC_Hisya(process.Sengo, process);
                        Button_Koma[] kmE = komas.Kyokumen.KmE_Hisya(process.Sengo, process);
                        Button_Koma[] kmG = komas.Kyokumen.KmG_Hisya(process.Sengo, process);

                        if (src.ExistsIn(KomanoKidou.SrcA_Hisya(process.Sengo, process)))
                        {
                            //----------
                            // Ａにいた
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.None;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(KomanoKidou.SrcC_Hisya(process.Sengo, process)))
                        {
                            //----------
                            // Ｃにいた
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(KomanoKidou.SrcE_Hisya(process.Sengo, process)))
                        {
                            //----------
                            // Ｅにいた
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.None;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(KomanoKidou.SrcG_Hisya(process.Sengo, process)))
                        {
                            //----------
                            // Ｇにいた
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                        }

                        // 「右」解除： Ｇに競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kmG)) { migiHidari = MigiHidari.None; }

                        // 「左」解除： Ｃに競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kmC)) { migiHidari = MigiHidari.None; }

                        // 「上」解除： Ａ、Ｃ、Ｇに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kmA) && process.NeverOnaji(kmC) && process.NeverOnaji(kmG)) { agaruHiku = AgaruHiku.None; }

                        // 「引」解除： Ｃ、Ｅ、Ｇに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kmC) && process.NeverOnaji(kmE) && process.NeverOnaji(kmG)) { agaruHiku = AgaruHiku.None; }

                        // 「寄」解除： Ａ、Ｅに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Yoru && process.NeverOnaji(kmA) && process.NeverOnaji(kmE)) { agaruHiku = AgaruHiku.None; }

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.None;
                        }
                    }
                    break;
                #endregion

                #region 角
                //************************************************************
                // 角
                //************************************************************
                case KomaSyurui.KAKU:
                    {
                        //----------
                        // 競合駒マス(range masu)
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │H7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │B7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │H6│  │  │  │  │  │  │  │  │  │  │  │  │  │B6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │H5│  │  │  │  │  │  │  │  │  │  │  │B5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │H4│  │  │  │  │  │  │  │  │  │B4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │H3│  │  │  │  │  │  │  │B3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │H2│  │  │  │  │  │B2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │H1│  │  │  │B1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │H0│  │B0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │至│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │F0│  │D0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │F1│  │  │  │D1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │F2│  │  │  │  │  │D2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │F3│  │  │  │  │  │  │  │D3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │F4│  │  │  │  │  │  │  │  │  │D4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │F5│  │  │  │  │  │  │  │  │  │  │  │D5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │F6│  │  │  │  │  │  │  │  │  │  │  │  │  │D6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │F7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │D7│
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        List<MasuBango> srcB = KomanoKidou.SrcB_Kaku(process.Sengo, process);
                        List<MasuBango> srcD = KomanoKidou.SrcD_Kaku(process.Sengo, process);
                        List<MasuBango> srcF = KomanoKidou.SrcF_Kaku(process.Sengo, process);
                        List<MasuBango> srcH = KomanoKidou.SrcH_Kaku(process.Sengo, process);
                        MasuBango src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        Button_Koma[] kmB = komas.Kyokumen.KmB_Kaku(process.Sengo, process);
                        Button_Koma[] kmD = komas.Kyokumen.KmD_Kaku(process.Sengo, process);
                        Button_Koma[] kmF = komas.Kyokumen.KmF_Kaku(process.Sengo, process);
                        Button_Koma[] kmH = komas.Kyokumen.KmH_Kaku(process.Sengo, process);


                        if (src.ExistsIn(srcB))
                        {
                            //----------
                            // 移動前はＢだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            //----------
                            // 移動前はＤだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            //----------
                            // 移動前はＦだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcH))
                        {
                            //----------
                            // 移動前はＨだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                        }

                        // 「右」解除： Ｆ、Ｈに競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kmF, kmH)) { migiHidari = MigiHidari.None; }

                        // 「左」解除： Ｂ、Ｄに競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kmB, kmD)) { migiHidari = MigiHidari.None; }

                        // 「上」解除： Ｂ、Ｈに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kmB, kmH)) { agaruHiku = AgaruHiku.None; }

                        // 「引」解除： Ｄ、Ｆに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kmD, kmF)) { agaruHiku = AgaruHiku.None; }

                        // 「寄」解除： 角は寄れません。

                        //----------
                        // 成
                        //----------
                        if (false==process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.None;
                        }
                    }
                    break;
                #endregion

                #region 香
                //************************************************************
                // 香
                //************************************************************
                case KomaSyurui.KYO:
                    {
                        //  ┌─┬─┬─┐
                        //  │  │至│  │
                        //  ├─┼─┼─┤
                        //  │  │E0│  │
                        //  ├─┼─┼─┤
                        //  │  │E1│  │
                        //  ├─┼─┼─┤
                        //  │  │E2│  │
                        //  ├─┼─┼─┤
                        //  │  │E3│  │
                        //  ├─┼─┼─┤
                        //  │  │E4│  │
                        //  ├─┼─┼─┤
                        //  │  │E5│  │
                        //  ├─┼─┼─┤
                        //  │  │E6│  │
                        //  ├─┼─┼─┤
                        //  │  │E7│  │
                        //  └─┴─┴─┘
                        List<MasuBango> srcE = KomanoKidou.SrcE_Hisya(process.Sengo, process);
                        MasuBango src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        Button_Koma[] kmA = komas.Kyokumen.KmA_Hisya(process.Sengo, process);

                        if (src.ExistsIn(srcE))
                        {
                            //----------
                            // 移動前はＥだった
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                        }

                        // 「右」解除： 香に右はありません。
                        // 「左」解除： 香に左はありません。
                        // 「上」解除： 香に上はありません。
                        // 「引」解除： 香に引はありません。
                        // 「寄」解除： 香に寄はありません。

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.None;
                        }
                    }
                    break;
                #endregion

                #region 桂
                //************************************************************
                // 桂
                //************************************************************
                case KomaSyurui.KEI:
                    {
                        //----------
                        // 競合駒マス
                        //----------
                        //┌─┐　┌─┐
                        //│  │　│  │
                        //├─┼─┼─┤
                        //│　│  │  │
                        //├─┼─┼─┤
                        //│　│至│  │先手から見た図
                        //├─┼─┼─┤
                        //│　│  │  │
                        //├─┼─┼─┤
                        //│Ｊ│　│Ｉ│
                        //└─┘　└─┘
                        List<MasuBango> srcI = KomanoKidou.SrcI_Kei(process.Sengo, process);
                        List<MasuBango> srcJ = KomanoKidou.SrcJ_Kei(process.Sengo, process);
                        MasuBango src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        Button_Koma[] kmI = komas.Kyokumen.KmI_Kei(process.Sengo, process);
                        Button_Koma[] kmJ = komas.Kyokumen.KmJ_Kei(process.Sengo, process);

                        if (src.ExistsIn(srcI))
                        {
                            //----------
                            // 移動前はＩだった
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcJ))
                        {
                            //----------
                            // 移動前はＪだった
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                        }

                        // 「右」解除： Ｊに競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kmJ)) { migiHidari = MigiHidari.None; }

                        // 「左」解除： Ｉに競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kmI)) { migiHidari = MigiHidari.None; }

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.None;
                        }
                    }
                    break;
                #endregion

                #region 銀
                //************************************************************
                // 銀
                //************************************************************
                case KomaSyurui.GIN:
                    {
                        //----------
                        // 競合駒マス(range masu)
                        //----------
                        //┌─┬─┬─┐
                        //│Ｈ│  │Ｂ│
                        //├─┼─┼─┤
                        //│　│至│  │先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        List<MasuBango> srcB = KomanoKidou.SrcB_Oh(process.Sengo, process);
                        List<MasuBango> srcD = KomanoKidou.SrcD_Oh(process.Sengo, process);
                        List<MasuBango> srcE = KomanoKidou.SrcE_Oh(process.Sengo, process);
                        List<MasuBango> srcF = KomanoKidou.SrcF_Oh(process.Sengo, process);
                        List<MasuBango> srcH = KomanoKidou.SrcH_Oh(process.Sengo, process);
                        MasuBango src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        Button_Koma[] kmB = komas.Kyokumen.KmB_Oh(process.Sengo, process);
                        Button_Koma[] kmD = komas.Kyokumen.KmD_Oh(process.Sengo, process);
                        Button_Koma[] kmE = komas.Kyokumen.KmE_Oh(process.Sengo, process);
                        Button_Koma[] kmF = komas.Kyokumen.KmF_Oh(process.Sengo, process);
                        Button_Koma[] kmH = komas.Kyokumen.KmH_Oh(process.Sengo, process);

                        if (process.IsDaAction)
                        {
                            // 打と明示されていた
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                            daHyoji = DaHyoji.Visible;
                        }
                        else if (src.ExistsIn(srcB))
                        {
                            // 移動前はＢだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            // 移動前はＤだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            // 移動前はＥだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Sugu;
                            // 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            // 移動前はＦだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcH))
                        {
                            // 移動前はＨだった
                            //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            //if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                        }

                        // 「右」解除： ①Ｅ、Ｆ、Ｈのどこにも競合駒がなければ。
                        //              ②Ｈに競合駒がなく引があるなら。
                        //              ③Ｅ、Ｆに競合駒がなく上があるなら。
                        if (migiHidari == MigiHidari.Migi && (
                            process.NeverOnaji(kmE, kmF, kmH)
                            || (process.NeverOnaji(kmH) && agaruHiku == AgaruHiku.Hiku)
                            || (process.NeverOnaji(kmE, kmF) && agaruHiku == AgaruHiku.Agaru)
                            )) { migiHidari = MigiHidari.None; }

                        // 「左」解除： ①Ｂ、Ｄ、Ｅのどこにも競合駒がなければ。
                        //              ②Ｂに競合駒がなく引があるなら。
                        //              ③Ｄ、Ｅに競合駒がなく上があるなら。
                        if (migiHidari == MigiHidari.Hidari && (
                            process.NeverOnaji(kmB, kmD, kmE)
                            || (process.NeverOnaji(kmB) && agaruHiku == AgaruHiku.Hiku)
                            || (process.NeverOnaji(kmD, kmE) && agaruHiku == AgaruHiku.Agaru)
                            )) { migiHidari = MigiHidari.None; }

                        // 「直」解除： Ｄ、Ｆのどちらにも競合駒がなければ。
                        if (migiHidari == MigiHidari.Sugu && process.NeverOnaji(kmD, kmF)) { migiHidari = MigiHidari.None; }

                        // 「上」解除： Ｂ、Ｈのどこにも競合駒がなければ。また、直があるなら。
                        if (agaruHiku == AgaruHiku.Agaru && (process.NeverOnaji(kmB, kmH) || migiHidari == MigiHidari.Sugu)) { agaruHiku = AgaruHiku.None; }

                        // 「引」解除： ①Ｂ、Ｄ、Ｅ、Ｆのどこにも競合駒がなければ。
                        //              ②Ｄ、Ｅ、Ｆ、Ｈのどこにも競合駒がなければ。
                        //              ③Ｄに競合駒がなく、右があるなら。
                        //              ④Ｆに競合駒がなく、左があるなら。
                        if (agaruHiku == AgaruHiku.Hiku &&
                            (
                            process.NeverOnaji(kmB, kmD, kmE, kmF)
                            || process.NeverOnaji(kmD, kmE, kmF, kmH)
                            || (process.NeverOnaji(kmD) && migiHidari == MigiHidari.Migi)
                            || (process.NeverOnaji(kmF) && migiHidari == MigiHidari.Hidari)
                            )) { agaruHiku = AgaruHiku.None; }

                        // 「寄」解除： 銀は寄れません。

                        //----------
                        // 成
                        //----------
                        if (false == process.Nari && !process.IsDaAction && process.InAitejin())
                        {
                            //成の指定がなく、相手陣内に指したら、非成を明示。
                            nari = NariFunari.Funari;
                        }
                        else if (process.Nari)
                        {
                            nari = NariFunari.Nari;
                        }
                        else
                        {
                            nari = NariFunari.None;
                        }
                    }
                    break;
                #endregion

                #region △金、△と金、△成香、△成桂、△成銀
                //************************************************************
                // △金、△と金、△成香、△成桂、△成銀
                //************************************************************
                case KomaSyurui.KIN:
                case KomaSyurui.TOKIN:
                case KomaSyurui.NARIKYO:
                case KomaSyurui.NARIKEI:
                case KomaSyurui.NARIGIN:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //┌─┬─┬─┐
                        //│  │Ａ│  │
                        //├─┼─┼─┤
                        //│Ｇ│至│Ｃ│先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        List<MasuBango> srcA = KomanoKidou.SrcA_Oh(process.Sengo, process);
                        List<MasuBango> srcC = KomanoKidou.SrcC_Oh(process.Sengo, process);
                        List<MasuBango> srcD = KomanoKidou.SrcD_Oh(process.Sengo, process);
                        List<MasuBango> srcE = KomanoKidou.SrcE_Oh(process.Sengo, process);
                        List<MasuBango> srcF = KomanoKidou.SrcF_Oh(process.Sengo, process);
                        List<MasuBango> srcG = KomanoKidou.SrcG_Oh(process.Sengo, process);
                        MasuBango src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        Button_Koma[] kmA = komas.Kyokumen.KmA_Oh(process.Sengo, process);
                        Button_Koma[] kmC = komas.Kyokumen.KmC_Oh(process.Sengo, process);
                        Button_Koma[] kmD = komas.Kyokumen.KmD_Oh(process.Sengo, process);
                        Button_Koma[] kmE = komas.Kyokumen.KmE_Oh(process.Sengo, process);
                        Button_Koma[] kmF = komas.Kyokumen.KmF_Oh(process.Sengo, process);
                        Button_Koma[] kmG = komas.Kyokumen.KmG_Oh(process.Sengo, process);


                        //----------
                        // 移動前はＡだった
                        //----------
                        if (src.ExistsIn(srcA))
                        {
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.None;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        //----------
                        // 移動前はＣだった
                        //----------
                        else if (src.ExistsIn(srcC))
                        {
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        //----------
                        // 移動前はＤだった
                        //----------
                        else if (src.ExistsIn(srcF))
                        {
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        //----------
                        // 移動前はＥだった
                        //----------
                        else if (src.ExistsIn(srcE))
                        {
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Sugu;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        //----------
                        // 移動前はＦだった
                        //----------
                        else if (src.ExistsIn(srcD))
                        {
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        //----------
                        // 移動前はＧだった
                        //----------
                        else if (src.ExistsIn(srcG))
                        {
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        //----------
                        // どこからか飛んできた
                        //----------
                        else
                        {
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                        }


                        // 「右」解除： ①Ｅ、Ｆ、Ｇのどちらにも競合駒がなければ。
                        //              ②Ｇに競合駒がなく、寄があるなら。
                        //              ③上があり、Ｅ、Ｆのどちらにも競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && (
                            process.NeverOnaji(kmE, kmF, kmG)
                            || (process.NeverOnaji(kmG) && agaruHiku == AgaruHiku.Yoru)
                            || (AgaruHiku.Agaru == agaruHiku && process.NeverOnaji(kmE, kmF))
                            )) { migiHidari = MigiHidari.None; }

                        // 「左」解除： ①Ｃ、Ｄ、Ｅのどちらにも競合駒がなければ。
                        //              ②Ｃに競合駒がなく、寄があるなら。
                        //              ③上があり、Ｄ、Ｅのどちらにも競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && (
                            process.NeverOnaji(kmC, kmD, kmE)
                            || (process.NeverOnaji(kmC) && agaruHiku == AgaruHiku.Yoru)
                            || (AgaruHiku.Agaru == agaruHiku && process.NeverOnaji(kmD, kmE))
                            )) { migiHidari = MigiHidari.None; }

                        // 「直」解除： Ｄ、Ｆのどちらにも競合駒がなければ。
                        if (migiHidari == MigiHidari.Sugu && process.NeverOnaji(kmD, kmF)) { migiHidari = MigiHidari.None; }

                        // 「上」解除： ①Ａ、Ｃ、Ｇのどこにも競合駒がなければ。
                        //              ②直があるなら。
                        //              ③Ｃに競合駒がなく、右があるなら。
                        //              ④Ｇに競合駒がなく、左があるなら。
                        if (agaruHiku == AgaruHiku.Agaru &&
                            (
                            process.NeverOnaji(kmA, kmC, kmG)
                            || migiHidari == MigiHidari.Sugu
                            || process.NeverOnaji(kmC) && migiHidari == MigiHidari.Migi
                            || process.NeverOnaji(kmG) && migiHidari == MigiHidari.Hidari
                            )
                            ) { agaruHiku = AgaruHiku.None; }

                        // 「引」解除： Ｄ、Ｅ、Ｆのどこにも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kmD, kmE, kmF)) { agaruHiku = AgaruHiku.None; }

                        // 「寄」解除： ①Ａ、Ｄ、Ｅ、Ｆのどこにも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Yoru && process.NeverOnaji(kmA, kmD, kmE, kmF)) { agaruHiku = AgaruHiku.None; }


                        //----------
                        // 成れません
                        //----------
                        nari = NariFunari.None;
                    }
                    break;
                #endregion

                #region 王
                //************************************************************
                // 王
                //************************************************************
                case KomaSyurui.OH:
                    {
                        //----------
                        // 競合駒マス(range masu)
                        //----------
                        //┌─┬─┬─┐
                        //│Ｈ│Ａ│Ｂ│
                        //├─┼─┼─┤
                        //│Ｇ│至│Ｃ│先手から見た図
                        //├─┼─┼─┤
                        //│Ｆ│Ｅ│Ｄ│
                        //└─┴─┴─┘
                        //List<MasuBango> srcA = KomanoKidou.SrcA_Oh(process.Sengo, process);
                        //List<MasuBango> srcB = KomanoKidou.SrcB_Oh(process.Sengo, process);
                        //List<MasuBango> srcC = KomanoKidou.SrcC_Oh(process.Sengo, process);
                        //List<MasuBango> srcD = KomanoKidou.SrcD_Oh(process.Sengo, process);
                        //List<MasuBango> srcE = KomanoKidou.SrcE_Oh(process.Sengo, process);
                        //List<MasuBango> srcF = KomanoKidou.SrcF_Oh(process.Sengo, process);
                        //List<MasuBango> srcG = KomanoKidou.SrcG_Oh(process.Sengo, process);
                        //List<MasuBango> srcH = KomanoKidou.SrcH_Oh(process.Sengo, process);
                        //MasuBango src = process.Src();


                        migiHidari = MigiHidari.None;
                        agaruHiku = AgaruHiku.None;

                        //----------
                        // 王は成れません
                        //----------
                        nari = NariFunari.None;
                    }
                    break;
                #endregion

                #region 竜
                //************************************************************
                // 竜
                //************************************************************
                case KomaSyurui.RYU:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │  │  │  │  │  │  │  │  │A7│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │A3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │A1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｈ│A0│Ｂ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │G7│G6│G5│G4│G3│G2│G1│G0│至│C0│C1│C2│C3│C4│C5│C6│C7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｆ│E0│Ｄ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E1│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E2│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E3│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │  │  │  │  │E4│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E5│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E6│  │  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │  │E7│  │  │  │  │  │  │  │  │
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        List<MasuBango> srcA = KomanoKidou.SrcA_Hisya(process.Sengo, process);
                        List<MasuBango> srcB = KomanoKidou.SrcB_Oh(process.Sengo, process);
                        List<MasuBango> srcC = KomanoKidou.SrcC_Hisya(process.Sengo, process);
                        List<MasuBango> srcD = KomanoKidou.SrcD_Oh(process.Sengo, process);
                        List<MasuBango> srcE = KomanoKidou.SrcE_Hisya(process.Sengo, process);
                        List<MasuBango> srcF = KomanoKidou.SrcF_Oh(process.Sengo, process);
                        List<MasuBango> srcG = KomanoKidou.SrcG_Hisya(process.Sengo, process);
                        List<MasuBango> srcH = KomanoKidou.SrcH_Oh(process.Sengo, process);
                        MasuBango src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        Button_Koma[] kmA = komas.Kyokumen.KmA_Hisya(process.Sengo, process);
                        Button_Koma[] kmB = komas.Kyokumen.KmB_Oh(process.Sengo, process);
                        Button_Koma[] kmC = komas.Kyokumen.KmC_Hisya(process.Sengo, process);
                        Button_Koma[] kmD = komas.Kyokumen.KmD_Oh(process.Sengo, process);
                        Button_Koma[] kmE = komas.Kyokumen.KmE_Hisya(process.Sengo, process);
                        Button_Koma[] kmF = komas.Kyokumen.KmF_Oh(process.Sengo, process);
                        Button_Koma[] kmG = komas.Kyokumen.KmG_Hisya(process.Sengo, process);
                        Button_Koma[] kmH = komas.Kyokumen.KmH_Oh(process.Sengo, process);


                        if (src.ExistsIn(srcA))
                        {
                            //----------
                            // 移動前はＡだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.None;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcB))
                        {
                            //----------
                            // 移動前はＢだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcC))
                        {
                            //----------
                            // 移動前はＣだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            //----------
                            // 移動前はＤだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            //----------
                            // 移動前はＥだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.None;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            //----------
                            // 移動前はＦだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcG))
                        {
                            //----------
                            // 移動前はＧだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcH))
                        {
                            //----------
                            // 移動前はＨだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                        }

                        // 「右」解除： Ａ、Ｅ、Ｆ、Ｇ、Ｈに１つも競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kmA, kmE, kmF, kmG, kmH)) { migiHidari = MigiHidari.None; }

                        // 「左」解除： Ａ、Ｂ、Ｃ、Ｄ、Ｅに１つも競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kmA, kmB, kmC, kmD, kmE)) { migiHidari = MigiHidari.None; }

                        // 「上」解除： Ａ、Ｂ、Ｃ、Ｇ、Ｈに１つも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kmA, kmB, kmC, kmG, kmH)) { agaruHiku = AgaruHiku.None; }

                        // 「引」解除： Ｃ、Ｄ、Ｅ、Ｆ、Ｇに１つも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kmC, kmD, kmE, kmF, kmG)) { agaruHiku = AgaruHiku.None; }

                        // 「寄」解除： Ａ、Ｂ、Ｄ、Ｅ、Ｆ、Ｈに１つも競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Yoru && process.NeverOnaji(kmA, kmB, kmD, kmE, kmF, kmH)) { agaruHiku = AgaruHiku.None; }

                        //----------
                        // 成れません
                        //----------
                        nari = NariFunari.None;
                    }
                    break;
                #endregion

                #region 馬
                //************************************************************
                // 馬
                //************************************************************
                case KomaSyurui.UMA:
                    {
                        //----------
                        // 競合駒マス(pre masu)
                        //----------
                        //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┬─┐
                        //  │H7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │B7│
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │H6│  │  │  │  │  │  │  │  │  │  │  │  │  │B6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │H5│  │  │  │  │  │  │  │  │  │  │  │B5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │H4│  │  │  │  │  │  │  │  │  │B4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │  │H3│  │  │  │  │  │  │  │B3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │H2│  │  │  │  │  │B2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │H1│  │  │  │B1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │H0│Ａ│B0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │Ｇ│至│Ｃ│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │  │F0│Ｅ│D0│  │  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │  │F1│  │  │  │D1│  │  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │  │F2│  │  │  │  │  │D2│  │  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │  │  │F3│  │  │  │  │  │  │  │D3│  │  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //　│  │  │  │F4│  │  │  │  │  │  │  │  │  │D4│  │  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │  │F5│  │  │  │  │  │  │  │  │  │  │  │D5│  │  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │  │F6│  │  │  │  │  │  │  │  │  │  │  │  │  │D6│  │
                        //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┼─┤
                        //  │F7│  │  │  │  │  │  │  │  │  │  │  │  │  │  │  │D7│
                        //  └─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┴─┘
                        List<MasuBango> srcA = KomanoKidou.SrcA_Oh(process.Sengo, process);
                        List<MasuBango> srcB = KomanoKidou.SrcB_Kaku(process.Sengo, process);
                        List<MasuBango> srcC = KomanoKidou.SrcC_Oh(process.Sengo, process);
                        List<MasuBango> srcD = KomanoKidou.SrcD_Kaku(process.Sengo, process);
                        List<MasuBango> srcE = KomanoKidou.SrcE_Oh(process.Sengo, process);
                        List<MasuBango> srcF = KomanoKidou.SrcF_Kaku(process.Sengo, process);
                        List<MasuBango> srcG = KomanoKidou.SrcG_Oh(process.Sengo, process);
                        List<MasuBango> srcH = KomanoKidou.SrcH_Kaku(process.Sengo, process);
                        MasuBango src = process.Src();

                        //----------
                        // 競合駒
                        //----------
                        Button_Koma[] kmA = komas.Kyokumen.KmA_Oh(process.Sengo, process);
                        Button_Koma[] kmB = komas.Kyokumen.KmB_Kaku(process.Sengo, process);
                        Button_Koma[] kmC = komas.Kyokumen.KmC_Oh(process.Sengo, process);
                        Button_Koma[] kmD = komas.Kyokumen.KmD_Kaku(process.Sengo, process);
                        Button_Koma[] kmE = komas.Kyokumen.KmE_Oh(process.Sengo, process);
                        Button_Koma[] kmF = komas.Kyokumen.KmF_Kaku(process.Sengo, process);
                        Button_Koma[] kmG = komas.Kyokumen.KmG_Oh(process.Sengo, process);
                        Button_Koma[] kmH = komas.Kyokumen.KmH_Kaku(process.Sengo, process);


                        if (src.ExistsIn(srcB))
                        {
                            //----------
                            // 移動前はＢだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcD))
                        {
                            //----------
                            // 移動前はＤだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcF))
                        {
                            //----------
                            // 移動前はＦだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcH))
                        {
                            //----------
                            // 移動前はＨだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcA))
                        {
                            //----------
                            // 移動前はＡだった
                            //----------
                            agaruHiku = AgaruHiku.Hiku;
                            migiHidari = MigiHidari.None;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcC))
                        {
                            //----------
                            // 移動前はＣだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Migi;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcE))
                        {
                            //----------
                            // 移動前はＥだった
                            //----------
                            agaruHiku = AgaruHiku.Agaru;
                            migiHidari = MigiHidari.None;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else if (src.ExistsIn(srcG))
                        {
                            //----------
                            // 移動前はＧだった
                            //----------
                            agaruHiku = AgaruHiku.Yoru;
                            migiHidari = MigiHidari.Hidari;
                            // 打
                            if (process.IsDaAction) { daHyoji = DaHyoji.Visible; }
                        }
                        else
                        {
                            //----------
                            // どこからか飛んできた
                            //----------
                            agaruHiku = AgaruHiku.None;
                            migiHidari = MigiHidari.None;
                        }

                        // 「右」解除： Ａ、Ｅ、Ｆ、Ｇ、Ｈに競合駒がなければ。
                        if (migiHidari == MigiHidari.Migi && process.NeverOnaji(kmA, kmE, kmF, kmG, kmH)) { migiHidari = MigiHidari.None; }

                        // 「左」解除： Ａ、Ｂ、Ｃ、Ｄ、Ｅに競合駒がなければ。
                        if (migiHidari == MigiHidari.Hidari && process.NeverOnaji(kmA, kmB, kmC, kmD, kmE)) { migiHidari = MigiHidari.None; }

                        // 「上」解除： Ａ、Ｂ、Ｃ、Ｇ、Ｈに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Agaru && process.NeverOnaji(kmA, kmB, kmC, kmG, kmH)) { agaruHiku = AgaruHiku.None; }

                        // 「引」解除： Ｃ、Ｄ、Ｅ、Ｆ、Ｇに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Hiku && process.NeverOnaji(kmC, kmD, kmE, kmF, kmG)) { agaruHiku = AgaruHiku.None; }

                        // 「寄」解除： Ａ、Ｂ、Ｄ、Ｅ、Ｆ、Ｈに競合駒がなければ。
                        if (agaruHiku == AgaruHiku.Yoru && process.NeverOnaji(kmA, kmB, kmD, kmE, kmF, kmH)) { agaruHiku = AgaruHiku.None; }

                        //----------
                        // 成れません
                        //----------
                        nari = NariFunari.None;
                    }
                    break;
                #endregion

                #region エラー
                //************************************************************
                // エラー
                //************************************************************
                default:
                    {
                        if (Sengo.GOTE == process.Sengo)
                        {
                            //******************************
                            // △後手
                            //******************************
                        }
                        else
                        {
                            //******************************
                            // ▲先手
                            //******************************
                        }

                        migiHidari = MigiHidari.None;
                        agaruHiku = AgaruHiku.None;

                        //----------
                        // TODO: 移動前の駒が成る前かどうか
                        //----------
                        nari = NariFunari.None;
                    }
                    break;
                #endregion

            }

            View_Fugo fugo = new View_Fugo(
                process.SrcSyurui,//「▲２二角成」のとき、dstだと馬になってしまう。srcの角を使う。
                migiHidari,
                agaruHiku,
                nari,
                daHyoji
                );

            return fugo;
        }

    }
}
