﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Reflection;
using System.Text.RegularExpressions;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 「▲７六歩」といった記述を、棋譜データの持ち方に変換します。
    /// </summary>
    public abstract class Notation1Importer
    {

        /// <summary>
        /// テキスト形式の符号を、リストに変換します。
        /// 
        /// 再生、コマ送りで利用。
        /// </summary>
        /// <returns></returns>
        public static bool TranslateTextToKifu(string text, out string restText, out Kifu kifuForAppend, Kifu baseKifu, KomaCollection komas, GameState gameState)
        {
            bool successful = false;

            restText = text;
            kifuForAppend = new Kifu();

            //------------------------------------------------------------
            // リスト作成
            //------------------------------------------------------------
            //
            //      “同”と書いてあっても、そのまま取り込みます。
            //

            Regex regex = new Regex(
                @"([▲△]?)(?:([123456789１２３４５６７８９])([123456789１２３４５６７８９一二三四五六七八九]))?(同)?[\s　]*(歩|香|桂|銀|金|飛|角|王|玉|と|成香|成桂|成銀|竜|龍|馬)(右|左|直)?(寄|引|上)?(成|不成)?(打?)",
                RegexOptions.Singleline
            );

            MatchCollection mc = regex.Matches(text);
            foreach (Match m in mc)
            {
                if (0 < m.Groups.Count)
                {
                    successful = true;

                    // 残りのテキスト
                    restText = text.Substring(0, m.Index) + text.Substring(m.Index + m.Length, text.Length - (m.Index + m.Length));

                    //------------------------------
                    // 処理：      “同”　→　前の符号と同じ筋、段
                    //------------------------------
                    KomaSyurui srcSyurui;
                    MasuBango process;
                    //View_Fugo tukawanaiFugo;

                    Translator_TextToProcess.TranslateTextToProcess(
                        m.Groups[1].Value,  //▲△
                        m.Groups[2].Value,  //123…9、１２３…９、一二三…九
                        m.Groups[3].Value,  //123…9、１２３…９、一二三…九
                        m.Groups[4].Value,  // “同”
                        m.Groups[5].Value,  //(歩|香|桂|…
                        m.Groups[6].Value,           // 右|左…
                        m.Groups[7].Value,  // 上|引
                        m.Groups[8].Value, //成|不成
                        m.Groups[9].Value,  //打
                        out srcSyurui,
                        out process,
                        komas,
                        baseKifu,
                        gameState
                        );

                    //------------------------------
                    // 記録：      棋譜に符号を追加
                    //------------------------------
                    MasuBango noUse; // 使いません。

                    kifuForAppend.AddAndRefresh(process, out noUse);
                }

                // 最初の１件だけ処理して終わります。
                break;
            }

            restText = restText.Trim();

            return successful;
        }


    }


}
