﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 符号。
    /// 
    /// 筋、段、同、駒種類、成、右左、上引、成、打
    /// ▲５五角
    /// </summary>
    public class View_Fugo
    {

        #region プロパティー類
        //------------------------------------------------------------
        private KomaSyurui komaSyurui;

        /// <summary>
        /// 駒種類
        /// </summary>
        public KomaSyurui KomaSyurui
        {
            get
            {
                return this.komaSyurui;
            }
        }
        //------------------------------------------------------------
        private MigiHidari migiHidari;

        /// <summary>
        /// 右、左、直など
        /// </summary>
        public MigiHidari MigiHidari
        {
            get
            {
                return this.migiHidari;
            }
        }
        //------------------------------------------------------------
        private AgaruHiku agaruHiku;

        /// <summary>
        /// 上、引
        /// </summary>
        public AgaruHiku AgaruHiku
        {
            get
            {
                return this.agaruHiku;
            }
        }
        //------------------------------------------------------------
        private NariFunari nari;

        /// <summary>
        /// 成
        /// </summary>
        public NariFunari Nari
        {
            get
            {
                return this.nari;
            }
        }
        //------------------------------------------------------------
        private DaHyoji daHyoji;

        /// <summary>
        /// “打”表示。必ずしも持ち駒を打つタイミングとは一致しない。
        /// </summary>
        public DaHyoji DaHyoji
        {
            get
            {
                return this.daHyoji;
            }
        }
        //------------------------------------------------------------
        #endregion

        public View_Fugo(
            KomaSyurui komaSyurui, MigiHidari migiHidari, AgaruHiku agaruHiku, NariFunari nari, DaHyoji daHyoji)
        {
            this.komaSyurui = komaSyurui;
            this.migiHidari = migiHidari;
            this.agaruHiku = agaruHiku;
            this.nari = nari;
            this.daHyoji = daHyoji;
        }


        /// <summary>
        /// 文字表記：　棋譜の符号用
        /// </summary>
        /// <param name="douExpr">“同”表記に置き換えるなら真。</param>
        /// <returns></returns>
        public string ToString(bool douExpr, MasuBango masuBango, MasuBango previousMasu)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append(Converter.SengoToStr(masuBango.Sengo));

            //------------------------------
            // “同”か、“筋・段”か
            //------------------------------
            if (
                douExpr
                && null!=previousMasu
                && previousMasu.MatchMasu(masuBango)
                )
            {
                sb.Append("同");
            }
            else
            {
                sb.Append(Converter.IntToArabiaSuji(masuBango.Suji));
                sb.Append(Converter.IntToKanSuji(masuBango.Dan));
            }

            //------------------------------
            // “歩”とか。“全”ではなく“成銀”    ＜符号用＞
            //------------------------------
            sb.Append(Converter.SyuruiToFugoText(this.KomaSyurui));

            //------------------------------
            // “右”とか
            //------------------------------
            sb.Append(Converter.MigiHidariToStr(this.MigiHidari));

            //------------------------------
            // “寄”とか
            //------------------------------
            sb.Append(Converter.AgaruHikuToStr(this.AgaruHiku));

            //------------------------------
            // “成”とか
            //------------------------------
            sb.Append(Converter.NariToStr(this.Nari));

            //------------------------------
            // “打”とか
            //------------------------------
            sb.Append(Converter.BoolToDa(this.DaHyoji));

            return sb.ToString();
        }

    }
}
