﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 駒の操作
    /// </summary>
    public abstract class Actions_Koma
    {

        /// <summary>
        /// 将棋盤オブジェクトを利用して、駒のx,y位置を変更します
        /// </summary>
        /// <param name="koma">駒</param>
        /// <param name="uc_Main">メインパネル</param>
        public static void RefreshKomaLocation( Button_Koma koma, Shogiban shogiban, Komadai[] komadaiArr, Uc_Main uc_Main )
        {
            switch (koma.Masu.Okiba)
            {
                case Okiba.SHOGIBAN:
                    koma.Bounds = new Rectangle(
                        shogiban.SujiToX(koma.Masu.Suji),
                        shogiban.DanToY(koma.Masu.Dan),
                        koma.Bounds.Width,
                        koma.Bounds.Height
                        );
                    break;

                case Okiba.SENTE_KOMADAI:
                    koma.Bounds = new Rectangle(
                        komadaiArr[0].SujiToX(koma.Masu.Suji),
                        komadaiArr[0].DanToY(koma.Masu.Dan),
                        koma.Bounds.Width,
                        koma.Bounds.Height
                        );
                    break;

                case Okiba.GOTE_KOMADAI:
                    koma.Bounds = new Rectangle(
                        komadaiArr[1].SujiToX(koma.Masu.Suji),
                        komadaiArr[1].DanToY(koma.Masu.Dan),
                        koma.Bounds.Width,
                        koma.Bounds.Height
                        );
                    break;
            }
        }


        /// <summary>
        /// 駒台の空いている場所を返します。無ければヌル。
        /// </summary>
        /// <param name="okiba">置き場</param>
        /// <param name="uc_Main">メインパネル</param>
        /// <returns>置ける場所</returns>
        public static MasuBango GetKomadaiSpace( Okiba okiba, Uc_Main uc_Main )
        {
            MasuBango result = null;

            int komadaiIndex = 0;
            if (okiba == Okiba.GOTE_KOMADAI)
            {
                komadaiIndex = 1;
            }


            bool[,] mat = new bool[uc_Main.KomadaiArr[komadaiIndex].LastDan, uc_Main.KomadaiArr[komadaiIndex].LastSuji];

            foreach (Button_Koma k in uc_Main.KomaCollection.Kyokumen.Items)
            {
                if (k.Masu.Okiba == okiba)
                {
                    mat[k.Masu.Dan - 1, k.Masu.Suji - 1] = true;
                }
            }

            for (int dan = 1; dan <= uc_Main.KomadaiArr[komadaiIndex].LastDan; dan++)
            {
                for (int suji = 1; suji <= uc_Main.KomadaiArr[komadaiIndex].LastSuji; suji++)
                {
                    if (!mat[dan-1, suji-1])
                    {
                        Sengo sengo;
                        switch (okiba)
                        {
                            case Okiba.GOTE_KOMADAI:
                                sengo = Sengo.GOTE;
                                break;
                            default:
                                sengo = Sengo.SENTE;
                                break;
                        }

                        result = new MasuBango(
                            sengo,
                            okiba,
                            Okiba.NO_USE,
                            suji, dan,
                            MasuBango.BUTTON_SUJI, MasuBango.BUTTON_DAN,
                            KomaSyurui.UNKNOWN,
                            KomaSyurui.UNKNOWN
                            );
                        goto gt_EndMethod;
                    }
                }
            }

        gt_EndMethod:
            return result;
        }

    }

}
