﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Xenon.KifuNarabe
{
    [Serializable]
    public class KomaCollection
    {

        #region 
        //------------------------------
        private Kyokumen kyokumen;

        public Kyokumen Kyokumen
        {
            get
            {
                return this.kyokumen;
            }
        }
        //------------------------------
        #endregion



        public KomaCollection()
        {
            this.kyokumen = new Kyokumen();

        }

        public List<Button_Koma> FindKomaByOkiba(Okiba okiba)
        {
            List<Button_Koma> list2 = new List<Button_Koma>();

            foreach (Button_Koma k in this.kyokumen.Items)
            {
                if (okiba == k.Masu.Okiba)
                {
                    list2.Add(k);
                }
            }

            return list2;
        }

        public List<Button_Koma> FindKomaByOkibaSengoSyurui(Okiba okiba, Sengo sengo, KomaSyurui syurui)
        {
            //------------------------------
            // 将棋盤の上の駒
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkiba(okiba);

            //------------------------------
            // 先後
            //------------------------------
            list1 = this.FindKomaBySengo(sengo, list1);

            //------------------------------
            // 種類
            //------------------------------
            list1 = this.FindKomaBySyurui(syurui, list1);

            return list1;
        }

        public List<Button_Koma> FindKomaBySengo(Sengo sengo, List<Button_Koma> list1)
        {
            List<Button_Koma> list2 = new List<Button_Koma>();

            foreach (Button_Koma btnKoma in list1)
            {
                if (sengo == btnKoma.Masu.Sengo)
                {
                    list2.Add(btnKoma);
                }
            }

            return list2;
        }

        public List<Button_Koma> FindKomaBySyurui(KomaSyurui syurui, List<Button_Koma> list1)
        {
            List<Button_Koma> list2 = new List<Button_Koma>();

            foreach (Button_Koma k in list1)
            {
                if (syurui == k.Masu.Syurui)
                {
                    list2.Add(k);
                }
            }

            return list2;
        }

        /// <summary>
        /// 駒の種類（成／不成を区別して扱います）を指定して、駒を全て検索します。無ければ空リスト。
        /// </summary>
        /// <param name="okiba"></param>
        /// <param name="syurui"></param>
        /// <param name="uc_Main"></param>
        /// <returns></returns>
        public List<Button_Koma> FindKomaBySyuruiAll(Okiba okiba, KomaSyurui syurui)
        {
            List<Button_Koma> komaList = new List<Button_Koma>();

            foreach (Button_Koma k in this.Kyokumen.Items)
            {
                if (k.Masu.Okiba == okiba
                    && k.Masu.Syurui == syurui)
                {
                    komaList.Add(k);
                }
            }

            return komaList;
        }

        /// <summary>
        /// 駒の種類（不成として扱います）を指定して、駒を検索します。無ければヌル。
        /// </summary>
        /// <param name="okiba"></param>
        /// <param name="syurui"></param>
        /// <param name="uc_Main"></param>
        /// <returns></returns>
        public Button_Koma FindKomaBySyuruiIgnoreCase(Okiba okiba, KomaSyurui syurui)
        {
            Button_Koma foundKoma = null;

            KomaSyurui syuruiFunariCase = Converter.SyuruiToFunariCase(syurui);

            foreach (Button_Koma koma in this.Kyokumen.Items)
            {
                if (koma.Masu.Okiba == okiba
                    && koma.Masu.ToFunariCase() == syuruiFunariCase)
                {
                    foundKoma = koma;
                    break;
                }
            }

            return foundKoma;
        }

        /// <summary>
        /// 移動直後の駒を取得。
        /// </summary>
        /// <returns>移動直後の駒。なければヌル</returns>
        public Button_Koma FindMovedKoma()
        {
            Button_Koma found = null;

            foreach (Button_Koma k in this.Kyokumen.Items)
            {
                if (k.Moved)
                {
                    found = k;
                }
            }

            return found;
        }


        /// <summary>
        /// 右竜、左竜、上がる竜、引く竜を検索します。該当がなければヌルです。
        /// 馬にも使えます。
        /// </summary>
        /// <param name="sengo"></param>
        /// <param name="syurui">竜か、馬のどちらかを指定してください。</param>
        /// <param name="migiRyu"></param>
        /// <param name="hidariRyu"></param>
        /// <param name="agaruRyu"></param>
        /// <param name="hikuRyu"></param>
        /// <param name="uc_Main"></param>
        public void FindRyu(Sengo sengo, KomaSyurui syurui, out Button_Koma migiRyu, out Button_Koma hidariRyu, out Button_Koma agaruRyu, out Button_Koma hikuRyu)
        {
            migiRyu = null;
            hidariRyu = null;
            agaruRyu = null;
            hikuRyu = null;

            //------------------------------
            // 全て検索
            //------------------------------
            //
            //      とりあえず、盤上の全ての竜を検索。
            //

            List<Button_Koma> komaList = this.FindKomaBySyuruiAll(Okiba.SHOGIBAN, syurui);

            if (2 != komaList.Count)
            {
                // 竜が２枚ではありませんでした
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                goto gt_EndMethod;
            }

            //------------------------------
            // １枚目の竜と、２枚目の竜の、筋比較
            //------------------------------
            if (komaList[0].Masu.Suji < komaList[1].Masu.Suji)
            {
                if (sengo == Sengo.GOTE)
                {
                    // 後手は左右逆になります。
                    migiRyu = komaList[1];
                    hidariRyu = komaList[0];
                }
                else
                {
                    migiRyu = komaList[0];
                    hidariRyu = komaList[1];
                }
            }
            else if (komaList[0].Masu.Suji > komaList[1].Masu.Suji)
            {
                if (sengo == Sengo.GOTE)
                {
                    // 後手は左右逆になります。
                    hidariRyu = komaList[1];
                    migiRyu = komaList[0];
                }
                else
                {
                    hidariRyu = komaList[0];
                    migiRyu = komaList[1];
                }
            }
            else
            {
                // 左、右に分けられませんでした
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
            }

            //------------------------------
            // １枚目の竜と、２枚目の竜の、段比較
            //------------------------------
            if (komaList[0].Masu.Dan < komaList[1].Masu.Dan)
            {
                if (sengo == Sengo.GOTE)
                {
                    // 後手は上下逆になります。
                    hikuRyu = komaList[1];
                    agaruRyu = komaList[0];
                }
                else
                {
                    hikuRyu = komaList[0];
                    agaruRyu = komaList[1];
                }
            }
            else if (komaList[0].Masu.Dan > komaList[1].Masu.Dan)
            {
                if (sengo == Sengo.GOTE)
                {
                    // 後手は上下逆になります。
                    agaruRyu = komaList[1];
                    hikuRyu = komaList[0];
                }
                else
                {
                    agaruRyu = komaList[0];
                    hikuRyu = komaList[1];
                }
            }
            else
            {
                // 上がる、引くに分けられませんでした
                //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
            }

        gt_EndMethod:
            ;
        }


        /// <summary>
        /// 選択されている駒を取得。
        /// </summary>
        /// <returns>選択されている駒。なければヌル</returns>
        public Button_Koma FindSelectedKoma()
        {
            Button_Koma found = null;

            foreach (Button_Koma k in this.Kyokumen.Items)
            {
                if (k.Selected)
                {
                    found = k;
                }
            }

            return found;
        }



        /// <summary>
        /// 歩を見つけます。
        /// </summary>
        public Button_Koma LookupFu(Sengo sengo, MasuBango ms)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）歩
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(
                Okiba.SHOGIBAN, sengo, KomaSyurui.FU);

            //----------
            // △歩
            //----------
            if (sengo == Sengo.GOTE)
            {
                foreach (Button_Koma k in list1)
                {
                    if (
                        ms.Suji == k.Masu.Suji
                        && ms.Dan - 1 == k.Masu.Dan
                    )
                    {
                        foundKoma = k;
                        break;
                    }
                }
            }
            //----------
            // ▲歩
            //----------
            else if (sengo == Sengo.SENTE)
            {
                foreach (Button_Koma k in list1)
                {
                    if (
                        ms.Suji == k.Masu.Suji
                        && ms.Dan + 1 == k.Masu.Dan
                    )
                    {
                        foundKoma = k;
                        break;
                    }
                }
            }


            return foundKoma;
        }



        /// <summary>
        /// 香を見つけます。
        /// </summary>
        public Button_Koma LookupKyo(Sengo sengo, MasuBango ms)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）香
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(
                Okiba.SHOGIBAN, sengo, KomaSyurui.KYO);

            if (sengo == Sengo.GOTE)
            {
                //----------
                // △香
                //----------
                foreach (Button_Koma k in list1)
                {
                    if (
                            (ms.Suji == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 2 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 3 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 4 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 5 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 6 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 7 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 8 == k.Masu.Dan)
                    )
                    {
                        foundKoma = k;
                        break;
                    }
                }
            }
            else if (sengo == Sengo.SENTE)
            {
                //----------
                // ▲香
                //----------
                foreach (Button_Koma k in list1)
                {
                    if (
                            (ms.Suji == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 2 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 3 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 4 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 5 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 6 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 7 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 8 == k.Masu.Dan)
                    )
                    {
                        foundKoma = k;
                        break;
                    }
                }
            }

            return foundKoma;
        }


        /// <summary>
        /// 桂を見つけます。
        /// </summary>
        public Button_Koma LookupKei(Sengo sengo, MasuBango ms, MigiHidari migiHidari)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）桂
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(
                Okiba.SHOGIBAN, sengo, KomaSyurui.KEI);


            //----------
            // 有りうる操作
            //----------
            bool migi = false; //右桂
            bool hidari = false; //左桂

            //----------
            // 右、左、その他
            //----------
            switch (migiHidari)
            {
                case MigiHidari.Migi:
                    migi = true;
                    break;
                case MigiHidari.Hidari:
                    hidari = true;
                    break;
                default:
                    migi = true;
                    hidari = true;
                    break;
            }

            if (sengo == Sengo.GOTE)
            {
                foreach (Button_Koma k in list1)
                {
                    //----------
                    // △桂   左上
                    //----------
                    if (
                        migi
                        &&
                        (ms.Suji + 1 == k.Masu.Suji && ms.Dan - 2 == k.Masu.Dan)
                        )
                    {
                        foundKoma = k;
                        break;
                    }

                    //----------
                    // △桂   右上
                    //----------
                    if (
                        hidari
                        &&
                        (ms.Suji - 1 == k.Masu.Suji && ms.Dan - 2 == k.Masu.Dan)
                        )
                    {
                        foundKoma = k;
                        break;
                    }
                }
            }
            else if (sengo == Sengo.SENTE)
            {
                foreach (Button_Koma k in list1)
                {
                    //----------
                    // ▲桂　左下
                    //----------
                    if (
                        hidari
                        &&
                        (ms.Suji + 1 == k.Masu.Suji && ms.Dan + 2 == k.Masu.Dan)
                        )
                    {
                        foundKoma = k;
                        break;
                    }

                    //----------
                    // ▲桂　右下
                    //----------
                    if (
                        migi
                        &&
                        (ms.Suji - 1 == k.Masu.Suji && ms.Dan + 2 == k.Masu.Dan)
                        )
                    {
                        foundKoma = k;
                        break;
                    }
                }
            }

            return foundKoma;
        }


        /// <summary>
        /// 銀を見つけます。
        /// </summary>
        public Button_Koma LookupGin(Sengo sengo, MasuBango ms, MigiHidari migiHidari, AgaruHiku agaruHiku)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）銀
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(Okiba.SHOGIBAN, sengo, KomaSyurui.GIN);

            foreach (Button_Koma k in list1)
            {
                //----------
                // 有りうる操作
                //----------
                bool migiHiku = false;      // 右引く
                bool hidariAgaru = false;   // 左上がる
                bool sugu = false;          // 直ぐ       ＜上がる＞
                bool migiAgaru = false;     // 右上がる
                bool hidariHiku = false;    // 左引く

                switch (migiHidari)
                {
                    //寄なし：　横には動けない
                    case MigiHidari.Migi:
                        migiHiku = true;
                        migiAgaru = true;
                        break;
                    case MigiHidari.Hidari:
                        hidariAgaru = true;
                        hidariHiku = true;
                        break;
                    case MigiHidari.Sugu:
                        sugu = true;
                        break;
                    default:
                        migiHiku = true;
                        migiAgaru = true;
                        sugu = true;
                        hidariAgaru = true;
                        hidariHiku = true;
                        break;
                }

                switch (agaruHiku)
                {
                    case AgaruHiku.Agaru:
                        migiAgaru = true;
                        hidariAgaru = true;
                        sugu = true;

                        migiHiku = false;
                        hidariHiku = false;
                        break;
                    case AgaruHiku.Hiku:
                        migiHiku = true;
                        hidariHiku = true;

                        migiAgaru = false;
                        hidariAgaru = false;
                        sugu = false;
                        break;
                    default:
                        break;
                }


                //----------
                // 真上
                //----------
                if (
                    (sengo == Sengo.GOTE && sugu)
                    &&
                    (ms.Suji == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 右上
                //----------
                else if (
                    ((sengo == Sengo.GOTE && hidariAgaru) || (sengo == Sengo.SENTE && migiHiku))
                    &&
                    (ms.Suji - 1 == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 右下
                //----------
                else if (
                    ((sengo == Sengo.GOTE && hidariHiku) || (sengo == Sengo.SENTE && migiAgaru))
                    &&
                    (ms.Suji - 1 == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真下
                //----------
                else if (
                    (sengo == Sengo.SENTE && sugu)
                    &&
                    (ms.Suji == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 左下
                //----------
                else if (
                    ((sengo == Sengo.GOTE && migiHiku) || (sengo == Sengo.SENTE && hidariAgaru))
                    &&
                    (ms.Suji + 1 == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 左上
                //----------
                else if (
                    ((sengo == Sengo.GOTE && migiAgaru) || (sengo == Sengo.SENTE && hidariHiku))
                    &&
                    (ms.Suji + 1 == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
            }


            return foundKoma;
        }

        /// <summary>
        /// 金を見つけます。
        /// </summary>
        public Button_Koma LookupKin(Sengo sengo, KomaSyurui komaSyurui, MasuBango ms, MigiHidari migiHidari, AgaruHiku agaruHiku)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）金
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(Okiba.SHOGIBAN, sengo, komaSyurui);

            foreach (Button_Koma km in list1)
            {
                //----------
                // 移動元ます    previous masu
                //----------
                MasuBango prA;
                MasuBango prC;
                MasuBango prD;
                MasuBango prE;
                MasuBango prF;
                MasuBango prG;


                bool kinA = false;          // 引く
                bool kinC = false;    // 真右寄る
                bool kinD = false;     // 右上がる
                bool kinE = false;          // 直ぐ       ＜上がる＞
                bool kinF = false;   // 左上がる
                bool kinG = false;  // 真左寄る

                if (sengo == Sengo.GOTE)
                {
                    //----------
                    // △金  他
                    //----------
                    //┌─┬─┬─┐
                    //│Ｄ│Ｅ│Ｆ│中心が移動先、英字は移動候補駒。後手なので天地逆。
                    //├─┼─┼─┤
                    //│Ｃ│　│Ｇ│
                    //├─┼─┼─┤
                    //│　│Ａ│　│
                    //└─┴─┴─┘

                    //----------
                    // 競合駒マス(pre masu)
                    //----------
                    prA = ms.Offset(sengo, 0, 1);//上
                    prC = ms.Offset(sengo, 1, 0);//右
                    prD = ms.Offset(sengo, 1, -1);//右下
                    prE = ms.Offset(sengo, 0, -1);//下
                    prF = ms.Offset(sengo, -1, -1);//左下
                    prG = ms.Offset(sengo, -1, 0);//左
                }
                else
                {
                    //----------
                    // ▲金  他
                    //----------
                    //┌─┬─┬─┐
                    //│Ｈ│Ａ│Ｂ│中心が移動先、英字は移動候補駒
                    //├─┼─┼─┤
                    //│Ｇ│　│Ｃ│
                    //├─┼─┼─┤
                    //│　│Ｅ│　│
                    //└─┴─┴─┘

                    //----------
                    // 競合駒マス(pre masu)
                    //----------
                    prA = ms.Offset(sengo, 0, -1);//上
                    prC = ms.Offset(sengo, -1, 0);//右
                    prD = ms.Offset(sengo, -1, 1);//右下
                    prE = ms.Offset(sengo, 0, 1);//下
                    prF = ms.Offset(sengo, 1, 1);//左下
                    prG = ms.Offset(sengo, 1, 0);//左
                }

                //----------
                // 競合駒
                //----------
                Button_Koma[] kmA = new Button_Koma[] { this.Kyokumen.AtMasu(km.Masu.Sengo, prA) };
                Button_Koma[] kmC = new Button_Koma[] { this.Kyokumen.AtMasu(km.Masu.Sengo, prC) };
                Button_Koma[] kmD = new Button_Koma[] { this.Kyokumen.AtMasu(km.Masu.Sengo, prD) };
                Button_Koma[] kmE = new Button_Koma[] { this.Kyokumen.AtMasu(km.Masu.Sengo, prE) };
                Button_Koma[] kmF = new Button_Koma[] { this.Kyokumen.AtMasu(km.Masu.Sengo, prF) };
                Button_Koma[] kmG = new Button_Koma[] { this.Kyokumen.AtMasu(km.Masu.Sengo, prG) };

                switch (migiHidari)
                {
                    case MigiHidari.Migi:
                        kinC = true;      // 右寄る
                        kinD = true;       // 右上がる
                        break;
                    case MigiHidari.Sugu:
                        kinE = true;            // 直ぐ
                        break;
                    case MigiHidari.Hidari:
                        kinF = true;     // 左上がる
                        kinG = true;    // 左寄る
                        break;
                    default:
                        kinA = true;            // 引く
                        kinC = true;      // 右寄る
                        kinD = true;       // 右上がる
                        kinE = true;            // 直ぐ
                        kinF = true;     // 左上がる
                        kinG = true;    // 左寄る
                        break;
                }

                switch (agaruHiku)
                {
                    case AgaruHiku.Yoru:
                        kinA = false;
                        kinD = false;
                        kinE = false;
                        kinF = false;
                        break;
                    case AgaruHiku.Agaru:
                        kinA = false;
                        kinC = false;
                        kinG = false;
                        break;
                    case AgaruHiku.Hiku:
                        kinC = false;
                        kinD = false;
                        kinE = false;
                        kinF = false;
                        kinG = false;
                        break;
                    default:
                        break;
                }

                //// 「右」解除： ①Ｅ、Ｆ、Ｇのどちらにも競合駒がなければ。
                ////              ②Ｇに競合駒がなく、寄があるなら。
                //if (migiHidari == MigiHidari.Migi && (
                //    km.NeverOnaji(kmE, kmF, kmG)
                //    || (km.NeverOnaji(kmG) && agaruHiku == AgaruHiku.Yoru)
                //    )) { migiHidari = MigiHidari.None; }

                //// 「左」解除： ①Ｃ、Ｄ、Ｅのどちらにも競合駒がなければ。
                ////              ②Ｃに競合駒がなく、寄があるなら。
                //if (migiHidari == MigiHidari.Hidari && (
                //    km.NeverOnaji(kmC, kmD, kmE)
                //    || (km.NeverOnaji(kmC) && agaruHiku == AgaruHiku.Yoru)
                //    )) { migiHidari = MigiHidari.None; }

                //// 「直」解除： Ｄ、Ｆのどちらにも競合駒がなければ。
                //if (migiHidari == MigiHidari.Sugu && km.NeverOnaji(kmD, kmF)) { migiHidari = MigiHidari.None; }

                //// 「上」解除： ①Ａ、Ｃ、Ｇのどこにも競合駒がなければ。
                ////              ②直があるなら。
                ////              ③Ｃに競合駒がなく、右があるなら。
                ////              ④Ｇに競合駒がなく、左があるなら。
                //if (agaruHiku == AgaruHiku.Agaru &&
                //    (
                //    km.NeverOnaji(kmA, kmC, kmG)
                //    || migiHidari == MigiHidari.Sugu
                //    || km.NeverOnaji(kmC) && migiHidari == MigiHidari.Migi
                //    || km.NeverOnaji(kmG) && migiHidari == MigiHidari.Hidari
                //    )
                //    ) { agaruHiku = AgaruHiku.None; }

                //// 「引」解除： Ｄ、Ｅ、Ｆのどこにも競合駒がなければ。
                //if (agaruHiku == AgaruHiku.Hiku && km.NeverOnaji(kmD, kmE, kmF)) { agaruHiku = AgaruHiku.None; }

                //// 「寄」解除： ①Ａ、Ｄ、Ｅ、Ｆのどこにも競合駒がなければ。
                //if (agaruHiku == AgaruHiku.Yoru && km.NeverOnaji(kmA, kmD, kmE, kmF)) { agaruHiku = AgaruHiku.None; }


                //----------
                // Ａ
                //----------
                Button_Koma matched = null;
                if (kinA && null!=(matched=km.Masu.MatchFirst(kmA)))
                {
                    foundKoma = matched;
                    break;
                }
                //----------
                // Ｃ
                //----------
                else if (kinC && null != (matched = km.Masu.MatchFirst(kmC)))
                {
                    foundKoma = matched;
                    break;
                }
                //----------
                // Ｄ
                //----------
                else if (kinD && null != (matched = km.Masu.MatchFirst(kmD)))
                {
                    foundKoma = matched;
                    break;
                }
                //----------
                // Ｅ
                //----------
                else if (kinE && null != (matched = km.Masu.MatchFirst(kmE)))
                {
                    foundKoma = matched;
                    break;
                }
                //----------
                // Ｆ
                //----------
                else if (kinF && null != (matched = km.Masu.MatchFirst(kmF)))
                {
                    foundKoma = matched;
                    break;
                }
                //----------
                // Ｇ
                //----------
                else if (kinG && null != (matched = km.Masu.MatchFirst(kmG)))
                {
                    foundKoma = matched;
                    break;
                }

            }

            return foundKoma;
        }

        /// <summary>
        /// 王を見つけます。
        /// </summary>
        public Button_Koma LookupOh(Sengo sengo, MasuBango ms, MigiHidari migiHidari, AgaruHiku agaruHiku)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）王
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(Okiba.SHOGIBAN, sengo, KomaSyurui.OH);


            foreach (Button_Koma k in list1)
            {
                bool hiku = false;          // 引く
                bool migiHiku = false;      // 右引く
                bool mamigiYoru = false;    // 真右寄る
                bool hidariAgaru = false;   // 左上がる
                bool sugu = false;          // 直ぐ       ＜上がる＞
                bool migiAgaru = false;     // 右上がる
                bool mahidariYoru = false;  // 真左寄る
                bool hidariHiku = false;    // 左引く

                //----------
                // △王、▲王
                //----------
                //
                //      １つしかないので左右、上引はありません。
                //
                hiku = true;
                migiHiku = true;
                mamigiYoru = true;
                hidariAgaru = true;
                sugu = true;
                migiAgaru = true;
                mahidariYoru = true;
                hidariHiku = true;

                //----------
                // 真上
                //----------
                if (
                    ((sengo == Sengo.GOTE && sugu) || (sengo == Sengo.SENTE && hiku))
                    &&
                    (ms.Suji == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 右上
                //----------
                else if (
                    ((sengo == Sengo.GOTE && hidariAgaru) || (sengo == Sengo.SENTE && migiHiku))
                    &&
                    (ms.Suji - 1 == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真右
                //----------
                else if (
                    ((sengo == Sengo.GOTE && mahidariYoru) || (sengo == Sengo.SENTE && mamigiYoru))
                    &&
                    (ms.Suji - 1 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 右下
                //----------
                else if (
                    ((sengo == Sengo.GOTE && hidariHiku) || (sengo == Sengo.SENTE && migiAgaru))
                    &&
                    (ms.Suji - 1 == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真下
                //----------
                else if (
                    ((sengo == Sengo.GOTE && sugu) || (sengo == Sengo.SENTE && hiku))
                    &&
                    (ms.Suji == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 左下
                //----------
                else if (
                    ((sengo == Sengo.GOTE && migiHiku) || (sengo == Sengo.SENTE && hidariAgaru))
                    &&
                    (ms.Suji + 1 == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真左
                //----------
                else if (
                    ((sengo == Sengo.GOTE && mamigiYoru) || (sengo == Sengo.SENTE && mahidariYoru))
                    &&
                    (ms.Suji + 1 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 左上
                //----------
                else if (
                    ((sengo == Sengo.GOTE && migiAgaru) || (sengo == Sengo.SENTE && hidariHiku))
                    &&
                    (ms.Suji + 1 == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
            }


            return foundKoma;
        }

        /// <summary>
        /// 飛を見つけます。
        /// </summary>
        public Button_Koma LookupHisya(Sengo sengo, MasuBango ms, MigiHidari migiHidari, AgaruHiku agaruHiku)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）飛
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(Okiba.SHOGIBAN, sengo, KomaSyurui.HISYA);

            foreach (Button_Koma km in list1)
            {
                MasuBango[] prA;
                MasuBango[] prC;
                MasuBango[] prE;
                MasuBango[] prG;

                if (sengo == Sengo.GOTE)
                {
                    //----------
                    // △飛
                    //----------
                    //                    Ｅ
                    //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                    //  │  │  │  │  │E3│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │E2│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │E1│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │E0│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │A0│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │A2│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │A3│  │  │  │  │
                    //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                    //                    Ａ

                    //----------
                    // 競合駒マス(pre masu)
                    //----------
                    prA = new MasuBango[8]; for (int i = 0; i < 8; i++) { prA[i] = ms.Offset(sengo, 0, i + 1); }
                    prC = new MasuBango[8]; for (int i = 0; i < 8; i++) { prC[i] = ms.Offset(sengo, i + 1, 0); }
                    prE = new MasuBango[8]; for (int i = 0; i < 8; i++) { prE[i] = ms.Offset(sengo, 0, -(i + 1)); }
                    prG = new MasuBango[8]; for (int i = 0; i < 8; i++) { prG[i] = ms.Offset(sengo, -(i + 1), 0); }
                }
                else
                {
                    //----------
                    // ▲飛
                    //----------
                    //                    Ａ
                    //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                    //  │  │  │  │  │A3│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │A2│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │A0│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │E0│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │E1│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │E2│  │  │  │  │
                    //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //  │  │  │  │  │E3│  │  │  │  │
                    //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                    //                    Ｅ

                    //----------
                    // 競合駒マス(pre masu)
                    //----------
                    prA = new MasuBango[8]; for (int i = 0; i < 8; i++) { prA[i] = ms.Offset(sengo, 0, -(i + 1)); }
                    prC = new MasuBango[8]; for (int i = 0; i < 8; i++) { prC[i] = ms.Offset(sengo, -(i + 1), 0); }
                    prE = new MasuBango[8]; for (int i = 0; i < 8; i++) { prE[i] = ms.Offset(sengo, 0, i + 1); }
                    prG = new MasuBango[8]; for (int i = 0; i < 8; i++) { prG[i] = ms.Offset(sengo, i + 1, 0); }
                }

                //----------
                // 競合駒
                //----------
                Button_Koma[] kmA = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmA[i] = this.Kyokumen.AtMasu(km.Masu.Sengo, prA[i]); }
                Button_Koma[] kmC = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmC[i] = this.Kyokumen.AtMasu(km.Masu.Sengo, prC[i]); }
                Button_Koma[] kmE = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmE[i] = this.Kyokumen.AtMasu(km.Masu.Sengo, prE[i]); }
                Button_Koma[] kmG = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmG[i] = this.Kyokumen.AtMasu(km.Masu.Sengo, prG[i]); }

                //----------
                // 移動元フラグ
                //----------
                bool fromA = false;  // 真上飛車
                bool fromC = false;    // 真右飛車
                bool fromE = false;    // 真下飛車
                bool fromG = false;  // 真左飛車

                switch (migiHidari)
                {
                    case MigiHidari.Migi:
                        fromC = true;    // 真右飛車
                        break;
                    case MigiHidari.Hidari:
                        fromG = true;  // 真左飛車
                        break;
                    default:
                        fromA = true;  // 真上飛車
                        fromC = true;    // 真右飛車
                        fromE = true;    // 真下飛車
                        fromG = true;  // 真左飛車
                        break;
                }

                switch (agaruHiku)
                {
                    case AgaruHiku.Yoru:
                        fromA = false;
                        fromE = false;
                        break;
                    case AgaruHiku.Agaru:
                        fromA = false;
                        fromC = false;
                        fromG = false;
                        break;
                    case AgaruHiku.Hiku:
                        fromC = false;    // 真右飛車
                        fromE = false;
                        fromG = false;  // 真左飛車
                        break;
                    default:
                        break;
                }

                //----------
                // Ａ
                //----------
                if (fromA && km.Masu.ExistsOnaji(kmA))
                {
                    foundKoma = km;
                    break;
                }
                //----------
                // Ｃ
                //----------
                else if (fromC && km.Masu.ExistsOnaji(kmC))
                {
                    foundKoma = km;
                    break;
                }
                //----------
                // Ｅ
                //----------
                else if (fromE && km.Masu.ExistsOnaji(kmE))
                {
                    foundKoma = km;
                    break;
                }
                //----------
                // Ｇ
                //----------
                else if (fromG && km.Masu.ExistsOnaji(kmG))
                {
                    foundKoma = km;
                    break;
                }
            }
            
            return foundKoma;
        }

        /// <summary>
        /// 角を見つけます。
        /// </summary>
        public Button_Koma LookupKaku(Sengo sengo, MasuBango ms, MigiHidari migiHidari, AgaruHiku agaruHiku)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）角
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(Okiba.SHOGIBAN, sengo, KomaSyurui.KAKU);

            foreach (Button_Koma km in list1)
            {
                //----------
                // 移動元ます    previous masu
                //----------
                MasuBango[] prB;
                MasuBango[] prD;
                MasuBango[] prF;
                MasuBango[] prH;

                if (sengo == Sengo.GOTE)
                {
                    //----------
                    // △角
                    //----------
                    //******************************
                    // △後手
                    //******************************
                    //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                    //　│D3│  │  │  │  │  │  │  │F3│
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │D2│  │  │  │  │  │F2│  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │D1│  │  │  │F1│  │  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │  │D0│  │F0│  │  │  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │  │  │  │  │  │  │  │中心が移動先、英字は移動可能元。後手なので天地逆。
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │  │B0│  │H0│  │  │  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │B1│  │  │  │H1│  │  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │B2│  │  │  │  │  │H2│  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│B3│  │  │  │  │  │  │  │H3│
                    //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                    //----------
                    // 競合駒マス(pre masu)
                    //----------
                    prB = new MasuBango[8]; for (int i = 0; i < 8; i++) { prB[i] = ms.Offset(sengo, i + 1, i + 1); }
                    prD = new MasuBango[8]; for (int i = 0; i < 8; i++) { prD[i] = ms.Offset(sengo, i + 1, -(i + 1)); }
                    prF = new MasuBango[8]; for (int i = 0; i < 8; i++) { prF[i] = ms.Offset(sengo, -(i + 1), -(i + 1)); }
                    prH = new MasuBango[8]; for (int i = 0; i < 8; i++) { prH[i] = ms.Offset(sengo, -(i + 1), i + 1); }
                }
                else
                {
                    //----------
                    // ▲角
                    //----------
                    //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                    //　│H3│  │  │  │  │  │  │  │B3│
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │H2│  │  │  │  │  │B2│  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │H1│  │  │  │B1│  │  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │  │H0│  │B0│  │  │  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │  │  │  │  │  │  │  │中心が移動先、英字は移動候補駒
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │  │F0│  │D0│  │  │  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │  │F1│  │  │  │D1│  │  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│  │F2│  │  │  │  │  │D2│  │
                    //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                    //　│F3│  │  │  │  │  │  │  │D3│
                    //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                    //----------
                    // 射程マス(range masu)
                    //----------
                    prB = new MasuBango[8]; for (int i = 0; i < 8; i++) { prB[i] = ms.Offset(sengo, -(i + 1), -(i + 1)); }
                    prD = new MasuBango[8]; for (int i = 0; i < 8; i++) { prD[i] = ms.Offset(sengo, -(i + 1), i + 1); }
                    prF = new MasuBango[8]; for (int i = 0; i < 8; i++) { prF[i] = ms.Offset(sengo, i + 1, i + 1); }
                    prH = new MasuBango[8]; for (int i = 0; i < 8; i++) { prH[i] = ms.Offset(sengo, i + 1, -(i + 1)); }
                }

                //----------
                // 射程駒
                //----------
                Button_Koma[] kmB = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmB[i] = this.Kyokumen.AtMasu(km.Masu.Sengo, prB[i]); }
                Button_Koma[] kmD = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmD[i] = this.Kyokumen.AtMasu(km.Masu.Sengo, prD[i]); }
                Button_Koma[] kmF = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmF[i] = this.Kyokumen.AtMasu(km.Masu.Sengo, prF[i]); }
                Button_Koma[] kmH = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmH[i] = this.Kyokumen.AtMasu(km.Masu.Sengo, prH[i]); }

                //----------
                // 移動元フラグ
                //----------
                bool fromB = false;  // 右上角
                bool fromD = false;    // 右下角
                bool fromF = false;    // 左下角
                bool fromH = false;  // 左上角

                switch (migiHidari)
                {
                    case MigiHidari.Migi:
                        fromB = true;  // 左上角
                        fromD = true;    // 左下角
                        break;
                    case MigiHidari.Hidari:
                        fromF = true;  // 右上角
                        fromH = true;    // 右下角
                        break;
                    default:
                        fromB = true;  // 右上角
                        fromD = true;    // 右下角
                        fromF = true;    // 左下角
                        fromH = true;  // 左上角
                        break;
                }

                switch (agaruHiku)
                {
                    case AgaruHiku.Agaru:
                        fromB = false;  // 右上角
                        fromH = false;  // 左上角
                        break;
                    case AgaruHiku.Hiku:
                        fromD = false;    // 右下角
                        fromF = false;    // 左下角
                        break;
                    default:
                        break;
                }

                //----------
                // Ｈ
                //----------
                if (fromH && km.Masu.ExistsOnaji(kmH))
                {
                    foundKoma = km;
                    break;
                }
                //----------
                // Ｂ
                //----------
                else if (fromB && km.Masu.ExistsOnaji(kmB))
                {
                    foundKoma = km;
                    break;
                }
                //----------
                // Ｆ
                //----------
                else if (fromF && km.Masu.ExistsOnaji(kmF))
                {
                    foundKoma = km;
                    break;
                }
                //----------
                // Ｄ
                //----------
                else if (fromD && km.Masu.ExistsOnaji(kmD))
                {
                    foundKoma = km;
                    break;
                }
            }


            return foundKoma;
        }

        /// <summary>
        /// 竜を見つけます。
        /// </summary>
        public Button_Koma LookupRyu(Sengo sengo, MasuBango ms, MigiHidari migiHidari, AgaruHiku agaruHiku)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）竜
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(Okiba.SHOGIBAN, sengo, KomaSyurui.RYU);

            foreach (Button_Koma k in list1)
            {
                //----------
                // 移動先フラグ
                //----------


                bool migiHiku = false;      // 右引く
                bool migiAgaru = false;     // 右上がる
                bool hidariAgaru = false;   // 左上がる
                bool hidariHiku = false;    // 左引く

                bool specialMaueHisya = false;  // 真上飛車
                bool specialMahidariHisya = false;  // 真左飛車
                bool specialMamigiHisya = false;    // 真右飛車
                bool specialMasitaHisya = false;    // 真下飛車

                bool specialRyu_Yoru = false;   //竜寄
                bool specialRyu_Migi = false;   //竜右
                bool specialRyu_Hidari = false; //竜左
                bool specialRyu_Agaru = false;  //竜上
                bool specialRyu_Hiku = false;   //竜引

                if (sengo == Sengo.GOTE)
                {
                    //----------
                    // △竜
                    //----------
                    //
                    //      竜は最大２枚しかないので、「直」は使わず、「右」「左」だけを使います。
                    //

                    switch (migiHidari)
                    {
                        case MigiHidari.Migi:
                            specialRyu_Migi = true;
                            break;
                        case MigiHidari.Hidari:
                            specialRyu_Hidari = true;
                            break;
                        //直なし
                        default:
                            break;
                    }

                    switch (agaruHiku)
                    {
                        case AgaruHiku.Yoru:
                            specialRyu_Yoru = true;
                            break;
                        case AgaruHiku.Agaru:
                            specialRyu_Agaru = true;
                            break;
                        case AgaruHiku.Hiku:
                            specialRyu_Hiku = true;
                            break;
                        default:
                            break;
                    }

                    if (!(specialRyu_Yoru
                        || specialRyu_Migi
                        || specialRyu_Hidari
                        || specialRyu_Agaru
                        || specialRyu_Hiku))
                    {
                        migiAgaru = true;
                        hidariAgaru = true;
                        hidariHiku = true;
                        migiHiku = true;
                        specialMaueHisya = true;  // 真上飛車
                        specialMahidariHisya = true;  // 真左飛車
                        specialMamigiHisya = true;    // 真右飛車
                        specialMasitaHisya = true;    // 真下飛車
                    }
                }
                else if (sengo == Sengo.SENTE)
                {
                    //----------
                    // ▲竜
                    //----------
                    //
                    //      竜は最大２枚しかないので、「直」は使わず、「右」「左」だけを使います。
                    //
                    switch (migiHidari)
                    {
                        case MigiHidari.Migi:
                            specialRyu_Migi = true;
                            break;
                        case MigiHidari.Hidari:
                            specialRyu_Hidari = true;
                            break;
                        //直なし
                        default:
                            break;
                    }

                    switch (agaruHiku)
                    {
                        case AgaruHiku.Yoru:
                            specialRyu_Yoru = true;
                            break;
                        case AgaruHiku.Agaru:
                            specialRyu_Agaru = true;
                            break;
                        case AgaruHiku.Hiku:
                            specialRyu_Hiku = true;
                            break;
                        default:
                            break;
                    }

                    if (!(specialRyu_Yoru
                        || specialRyu_Migi
                        || specialRyu_Hidari
                        || specialRyu_Agaru
                        || specialRyu_Hiku))
                    {
                        hidariHiku = true;
                        migiHiku = true;
                        migiAgaru = true;
                        hidariAgaru = true;
                        specialMaueHisya = true;  // 真上飛車
                        specialMahidariHisya = true;  // 真左飛車
                        specialMamigiHisya = true;    // 真右飛車
                        specialMasitaHisya = true;    // 真下飛車
                    }
                }


                //----------
                // 右上
                //----------
                if (
                    ((sengo == Sengo.GOTE && hidariAgaru) || (sengo == Sengo.SENTE && migiHiku))
                    &&
                    (ms.Suji - 1 == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 右下
                //----------
                else if (
                    ((sengo == Sengo.GOTE && hidariHiku) || (sengo == Sengo.SENTE && migiAgaru))
                    &&
                    (ms.Suji - 1 == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 左下
                //----------
                else if (
                    ((sengo == Sengo.GOTE && migiHiku) || (sengo == Sengo.SENTE && hidariAgaru))
                    &&
                    (ms.Suji + 1 == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 左上
                //----------
                else if (
                    ((sengo == Sengo.GOTE && migiAgaru) || (sengo == Sengo.SENTE && hidariHiku))
                    &&
                    (ms.Suji + 1 == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真上飛車
                //----------
                else if (
                    specialMaueHisya
                    &&
                    (
                            (ms.Suji == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 2 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 3 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 4 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 5 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 6 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 7 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan - 8 == k.Masu.Dan)
                    )
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真下飛車
                //----------
                else if (
                    specialMasitaHisya
                    &&
                    (
                            (ms.Suji == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 2 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 3 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 4 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 5 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 6 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 7 == k.Masu.Dan)
                        || (ms.Suji == k.Masu.Suji && ms.Dan + 8 == k.Masu.Dan)
                    )
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真左飛車
                //----------
                else if (
                    specialMahidariHisya
                    &&
                    (
                            (ms.Suji - 1 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji - 2 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji - 3 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji - 4 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji - 5 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji - 6 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji - 7 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji - 8 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                    )
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真右飛車
                //----------
                else if (
                    specialMamigiHisya
                    &&
                    (
                            (ms.Suji + 1 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji + 2 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji + 3 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji + 4 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji + 5 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji + 6 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji + 7 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                        || (ms.Suji + 8 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                    )
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 竜：   寄り
                //----------
                else if (specialRyu_Yoru)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.RYU;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は右竜か？
                    if (null != migiRyu && migiRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、右竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                    else if (null != hidariRyu && hidariRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、左竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                    else
                    {
                        // この竜は、右竜でも左竜でも　ありませんでした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        // エラー
                    }
                }
                //----------
                // 竜：   右
                //----------
                else if (specialRyu_Migi)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.RYU;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は右竜か？
                    if (null != migiRyu && migiRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、右竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                }
                //----------
                // 竜：   左
                //----------
                else if (specialRyu_Hidari)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.RYU;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は左竜か？
                    if (null != hidariRyu && hidariRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、左竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                }
                //----------
                // 竜：   上がる
                //----------
                else if (specialRyu_Agaru)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.RYU;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は上がる竜か？
                    if (null != agaruRyu && agaruRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、上がる竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                }
                //----------
                // 竜：   引く
                //----------
                else if (specialRyu_Hiku)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.RYU;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は引く竜か？
                    if (null != hikuRyu && hikuRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、引く竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                }
            }

            return foundKoma;
        }

        /// <summary>
        /// 馬を見つけます。
        /// </summary>
        public Button_Koma LookupUma(Sengo sengo, MasuBango ms, MigiHidari migiHidari, AgaruHiku agaruHiku)
        {
            Button_Koma foundKoma = null;

            //------------------------------
            // 盤上の全ての▲（でなければ△）馬
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkibaSengoSyurui(Okiba.SHOGIBAN, sengo, KomaSyurui.UMA);


            foreach (Button_Koma k in list1)
            {
                bool hiku = false;          // 引く
                bool mamigiYoru = false;    // 真右寄る
                bool sugu = false;          // 直ぐ       ＜上がる＞
                bool mahidariYoru = false;  // 真左寄る

                bool specialHidariueKaku = false;  // 左上角
                bool specialMigiueKaku = false;  // 右上角
                bool specialHidarisitaKaku = false;    // 左下角
                bool specialMigisitaKaku = false;    // 右下角

                bool specialUma_Yoru = false;   //馬寄
                bool specialUma_Migi = false;   //馬右
                bool specialUma_Hidari = false; //馬左
                bool specialUma_Agaru = false;  //馬上
                bool specialUma_Hiku = false;   //馬引

                if (sengo == Sengo.GOTE)
                {
                    //----------
                    // △馬
                    //----------
                    //
                    //      馬は最大２枚しかないので、「直」は使わず、「右」「左」だけを使います。
                    //
                    switch (migiHidari)
                    {
                        case MigiHidari.Migi:
                            specialUma_Migi = true;
                            break;
                        case MigiHidari.Hidari:
                            specialUma_Hidari = true;
                            break;
                        //直なし
                        default:
                            break;
                    }

                    switch (agaruHiku)
                    {
                        case AgaruHiku.Yoru:
                            specialUma_Yoru = true;
                            break;
                        case AgaruHiku.Agaru:
                            specialUma_Agaru = true;
                            break;
                        case AgaruHiku.Hiku:
                            specialUma_Hiku = true;
                            break;
                        default:
                            break;
                    }

                    if (!(specialUma_Yoru
                            || specialUma_Migi
                            || specialUma_Hidari
                            || specialUma_Agaru
                            || specialUma_Hiku))
                    {
                        hiku = true;
                        mamigiYoru = true;
                        sugu = true;
                        mahidariYoru = true;
                        specialHidariueKaku = true;  // 左上角
                        specialMigiueKaku = true;  // 右上角
                        specialHidarisitaKaku = true;    // 左下角
                        specialMigisitaKaku = true;    // 右下角
                    }
                }
                else if (sengo == Sengo.SENTE)
                {
                    //----------
                    // ▲馬
                    //----------
                    //
                    //      馬は最大２枚しかないので、「直」は使わず、「右」「左」だけを使います。
                    //
                    switch (migiHidari)
                    {
                        case MigiHidari.Migi:
                            specialUma_Migi = true;
                            break;
                        case MigiHidari.Hidari:
                            specialUma_Hidari = true;
                            break;
                        //直なし
                        default:
                            break;
                    }

                    switch (agaruHiku)
                    {
                        case AgaruHiku.Yoru:
                            specialUma_Yoru = true;
                            break;
                        case AgaruHiku.Agaru:
                            specialUma_Agaru = true;
                            break;
                        case AgaruHiku.Hiku:
                            specialUma_Hiku = true;
                            break;
                        default:
                            break;
                    }

                    if (!(specialUma_Yoru
                            || specialUma_Migi
                            || specialUma_Hidari
                            || specialUma_Agaru
                            || specialUma_Hiku))
                    {
                        hiku = true;
                        mamigiYoru = true;
                        sugu = true;
                        mahidariYoru = true;
                        specialHidariueKaku = true;  // 左上角
                        specialMigiueKaku = true;  // 右上角
                        specialHidarisitaKaku = true;    // 左下角
                        specialMigisitaKaku = true;    // 右下角
                    }
                }


                //----------
                // 真上
                //----------
                if (
                    ((sengo == Sengo.GOTE && sugu) || (sengo == Sengo.SENTE && hiku))
                    &&
                    (ms.Suji == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真右
                //----------
                else if (
                    ((sengo == Sengo.GOTE && mahidariYoru) || (sengo == Sengo.SENTE && mamigiYoru))
                    &&
                    (ms.Suji - 1 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真下
                //----------
                else if (
                    ((sengo == Sengo.GOTE && sugu) || (sengo == Sengo.SENTE && hiku))
                    &&
                    (ms.Suji == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 真左
                //----------
                else if (
                    ((sengo == Sengo.GOTE && mamigiYoru) || (sengo == Sengo.SENTE && mahidariYoru))
                    &&
                    (ms.Suji + 1 == k.Masu.Suji && ms.Dan == k.Masu.Dan)
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 左上角
                //----------
                else if (
                    specialHidariueKaku
                    &&
                    (
                            (ms.Suji + 1 == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                        || (ms.Suji + 2 == k.Masu.Suji && ms.Dan - 2 == k.Masu.Dan)
                        || (ms.Suji + 3 == k.Masu.Suji && ms.Dan - 3 == k.Masu.Dan)
                        || (ms.Suji + 4 == k.Masu.Suji && ms.Dan - 4 == k.Masu.Dan)
                        || (ms.Suji + 5 == k.Masu.Suji && ms.Dan - 5 == k.Masu.Dan)
                        || (ms.Suji + 6 == k.Masu.Suji && ms.Dan - 6 == k.Masu.Dan)
                        || (ms.Suji + 7 == k.Masu.Suji && ms.Dan - 7 == k.Masu.Dan)
                        || (ms.Suji + 8 == k.Masu.Suji && ms.Dan - 8 == k.Masu.Dan)
                    )
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 右上角
                //----------
                else if (
                    specialMigiueKaku
                    &&
                    (
                            (ms.Suji - 1 == k.Masu.Suji && ms.Dan - 1 == k.Masu.Dan)
                        || (ms.Suji - 2 == k.Masu.Suji && ms.Dan - 2 == k.Masu.Dan)
                        || (ms.Suji - 3 == k.Masu.Suji && ms.Dan - 3 == k.Masu.Dan)
                        || (ms.Suji - 4 == k.Masu.Suji && ms.Dan - 4 == k.Masu.Dan)
                        || (ms.Suji - 5 == k.Masu.Suji && ms.Dan - 5 == k.Masu.Dan)
                        || (ms.Suji - 6 == k.Masu.Suji && ms.Dan - 6 == k.Masu.Dan)
                        || (ms.Suji - 7 == k.Masu.Suji && ms.Dan - 7 == k.Masu.Dan)
                        || (ms.Suji - 8 == k.Masu.Suji && ms.Dan - 8 == k.Masu.Dan)
                    )
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 左下角
                //----------
                else if (
                    specialHidarisitaKaku
                    &&
                    (
                            (ms.Suji + 1 == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                        || (ms.Suji + 2 == k.Masu.Suji && ms.Dan + 2 == k.Masu.Dan)
                        || (ms.Suji + 3 == k.Masu.Suji && ms.Dan + 3 == k.Masu.Dan)
                        || (ms.Suji + 4 == k.Masu.Suji && ms.Dan + 4 == k.Masu.Dan)
                        || (ms.Suji + 5 == k.Masu.Suji && ms.Dan + 5 == k.Masu.Dan)
                        || (ms.Suji + 6 == k.Masu.Suji && ms.Dan + 6 == k.Masu.Dan)
                        || (ms.Suji + 7 == k.Masu.Suji && ms.Dan + 7 == k.Masu.Dan)
                        || (ms.Suji + 8 == k.Masu.Suji && ms.Dan + 8 == k.Masu.Dan)
                    )
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 右下角
                //----------
                else if (
                    specialMigisitaKaku
                    &&
                    (
                            (ms.Suji - 1 == k.Masu.Suji && ms.Dan + 1 == k.Masu.Dan)
                        || (ms.Suji - 2 == k.Masu.Suji && ms.Dan + 2 == k.Masu.Dan)
                        || (ms.Suji - 3 == k.Masu.Suji && ms.Dan + 3 == k.Masu.Dan)
                        || (ms.Suji - 4 == k.Masu.Suji && ms.Dan + 4 == k.Masu.Dan)
                        || (ms.Suji - 5 == k.Masu.Suji && ms.Dan + 5 == k.Masu.Dan)
                        || (ms.Suji - 6 == k.Masu.Suji && ms.Dan + 6 == k.Masu.Dan)
                        || (ms.Suji - 7 == k.Masu.Suji && ms.Dan + 7 == k.Masu.Dan)
                        || (ms.Suji - 8 == k.Masu.Suji && ms.Dan + 8 == k.Masu.Dan)
                    )
                )
                {
                    foundKoma = k;
                    break;
                }
                //----------
                // 馬：   寄り
                //----------
                else if (specialUma_Yoru)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.UMA;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は右竜か？
                    if (null != migiRyu && migiRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、右竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                    else if (null != hidariRyu && hidariRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、左竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                    else
                    {
                        // この竜は、右竜でも左竜でも　ありませんでした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        // エラー
                    }
                }
                //----------
                // 馬：   右
                //----------
                else if (specialUma_Migi)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.UMA;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は右竜か？
                    if (null != migiRyu && migiRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、右竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                }
                //----------
                // 馬：   左
                //----------
                else if (specialUma_Hidari)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.UMA;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は左竜か？
                    if (null != hidariRyu && hidariRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、左竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                }
                //----------
                // 馬：   上がる
                //----------
                else if (specialUma_Agaru)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.UMA;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は上がる竜か？
                    if (null != agaruRyu && agaruRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、上がる竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                }
                //----------
                // 馬：   引く
                //----------
                else if (specialUma_Hiku)
                {
                    KomaSyurui syuruiRyuma = KomaSyurui.UMA;

                    // 右竜、左竜検索
                    Button_Koma migiRyu;
                    Button_Koma hidariRyu;
                    Button_Koma agaruRyu;
                    Button_Koma hikuRyu;
                    this.FindRyu(sengo, syuruiRyuma, out migiRyu, out hidariRyu, out agaruRyu, out hikuRyu);

                    // この竜は引く竜か？
                    if (null != hikuRyu && hikuRyu.Masu.MatchMasu(k.Masu))
                    {
                        // この竜は、引く竜でした
                        //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

                        foundKoma = k;
                        break;
                    }
                }
            }

            return foundKoma;
        }


        /// <summary>
        /// 指定の場所に行ける駒を返します。無ければヌル。
        /// </summary>
        /// <param name="okiba">置き場</param>
        /// <param name="suji">筋</param>
        /// <param name="dan">段</param>
        /// <param name="uc_Main">メインパネル</param>
        /// <returns>駒</returns>
        public Button_Koma FindKomaByDestination(Sengo sengo, MasuBango ms, KomaSyurui syurui, MigiHidari migiHidari, AgaruHiku agaruHiku)
        {

            //------------------------------
            // 将棋盤の上の駒
            //------------------------------
            List<Button_Koma> list1 = this.FindKomaByOkiba(Okiba.SHOGIBAN);

            //------------------------------
            // 先後
            //------------------------------
            list1 = this.FindKomaBySengo(sengo, list1);

            //------------------------------
            // 種類
            //------------------------------
            list1 = this.FindKomaBySyurui(syurui, list1);


            Button_Koma foundKoma = null;


            switch (syurui)
            {
                //**********
                // 歩
                //**********
                case KomaSyurui.FU:
                    foundKoma = this.LookupFu(sengo, ms);
                    break;

                //**********
                // 香
                //**********
                case KomaSyurui.KYO:
                    foundKoma = this.LookupKyo(sengo, ms);
                    break;

                //**********
                // 桂
                //**********
                case KomaSyurui.KEI:
                    foundKoma = this.LookupKei(sengo, ms, migiHidari);
                    break;

                //************************************************************
                // 銀
                //************************************************************
                case KomaSyurui.GIN:
                    foundKoma = this.LookupGin(sengo, ms, migiHidari, agaruHiku);
                    break;

                //************************************************************
                // 金、と、杏、圭、全
                //************************************************************
                case KomaSyurui.KIN:
                case KomaSyurui.TOKIN:
                case KomaSyurui.NARIKYO:
                case KomaSyurui.NARIKEI:
                case KomaSyurui.NARIGIN:
                    foundKoma = this.LookupKin(sengo, syurui, ms, migiHidari, agaruHiku);
                    break;

                //************************************************************
                // 王
                //************************************************************
                case KomaSyurui.OH:
                    foundKoma = this.LookupOh(sengo, ms, migiHidari, agaruHiku);
                    break;

                //************************************************************
                // 飛
                //************************************************************
                case KomaSyurui.HISYA:
                    foundKoma = this.LookupHisya(sengo, ms, migiHidari, agaruHiku);
                    break;

                //************************************************************
                // 角
                //************************************************************
                case KomaSyurui.KAKU:
                    foundKoma = this.LookupKaku(sengo, ms, migiHidari, agaruHiku);
                    break;

                //************************************************************
                // 竜
                //************************************************************
                case KomaSyurui.RYU:
                    foundKoma = this.LookupRyu(sengo, ms, migiHidari, agaruHiku);
                    break;

                //************************************************************
                // 馬
                //************************************************************
                case KomaSyurui.UMA:
                    foundKoma = this.LookupUma(sengo, ms, migiHidari, agaruHiku);
                    break;

                //************************************************************
                // それ以外
                //************************************************************
                default:
                    break;
            }


            if (null == foundKoma)
            {
                //----------
                // 駒台を確認
                //----------
                //
                //      駒台の持ち駒を打つのが分かりきっていて、“打”が省かれている可能性
                //
                foreach (Button_Koma k in this.Kyokumen.Items)
                {
                    if (
                        (Okiba.SENTE_KOMADAI == k.Masu.Okiba || Okiba.GOTE_KOMADAI == k.Masu.Okiba)
                        )
                    {
                        foundKoma = this.FindKomaBySyuruiIgnoreCase(
                            Converter.SengoToKomadaiOkiba(sengo),
                            syurui);
                    }
                }
            }

            return foundKoma;
        }



        /// <summary>
        /// 各駒の、移動済フラグを解除
        /// </summary>
        public void ClearAllSelectedFlag()
        {
            foreach (Button_Koma k in this.Kyokumen.Items)
            {
                k.Selected = false;
                k.SelectFirstTouch = false;
            }
        }


        /// <summary>
        /// 各駒の、移動済フラグを解除
        /// </summary>
        public void ClearAllMoverdFlag()
        {
            foreach (Button_Koma k in this.Kyokumen.Items)
            {
                k.Moved = false;
            }
        }



    }
}
