﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;



namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 駒の軌道。
    /// </summary>
    public abstract class KomanoKidou
    {


        public static List<MasuBango> SrcA_Hisya(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll = new List<MasuBango>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │至│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A7│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //　                  Ａ

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, 0, i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //　                  Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A7│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │至│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, 0, -(i + 1))); }
            }

            return srcAll;
        }


        public static List<MasuBango> SrcC_Hisya(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll = new List<MasuBango>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C7│C6│C5│C4│C3│C2│C1│C0│至│
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, i + 1, 0)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│G0│G1│G2│G3│G4│G5│G6│G7│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, -(i + 1), 0)); }
            }

            return srcAll;
        }


        public static List<MasuBango> SrcE_Hisya(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll = new List<MasuBango>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //　                  Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E7│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │至│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, 0, -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │至│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E0│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E4│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E5│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E6│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E7│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //　                  Ｅ

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, 0, i + 1)); }
            }

            return srcAll;
        }


        public static List<MasuBango> SrcG_Hisya(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll = new List<MasuBango>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│G0│G1│G2│G3│G4│G5│G6│G7│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, i + 1, 0)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G7│G6│G5│G4│G3│G2│G1│G0│至│
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │  │  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, -(i + 1), 0)); }
            }

            return srcAll;
        }


        public static List<MasuBango> SrcB_Kaku(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll = new List<MasuBango>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│  │  │  │  │  │  │  │  │至│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │B0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │B2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │B3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B7│  │  │  │  │  │  │  │  │
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, i + 1, i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│  │  │  │  │  │  │  │  │B7│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │B6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │B5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │B4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │B3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│  │  │  │  │  │  │  │  │
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, -(i + 1), -(i + 1))); }
            }

            return srcAll;
        }


        public static List<MasuBango> SrcD_Kaku(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll = new List<MasuBango>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│D7│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │D3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │D2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │D0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │至│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, i + 1, -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //　┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│至│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │D3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │D4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │D5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │D6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │D7│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, -(i + 1), i + 1)); }
            }

            return srcAll;
        }


        public static List<MasuBango> SrcF_Kaku(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll = new List<MasuBango>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│  │  │  │  │  │  │  │  │F7│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │F6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │F5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │F4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │F3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│至│  │  │  │  │  │  │  │  │
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, -(i + 1), -(i + 1))); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│  │  │  │  │  │  │  │  │至│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │F0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │F2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │F3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F7│  │  │  │  │  │  │  │  │
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, i + 1, i + 1)); }
            }

            return srcAll;
        }


        public static List<MasuBango> SrcH_Kaku(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll = new List<MasuBango>();

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //　┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│至│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H0│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H2│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │H3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │H4│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │H5│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │H6│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │H7│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, -(i + 1), i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //　│H7│  │  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H6│  │  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H5│  │  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H4│  │  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │H3│  │  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │H2│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │H0│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │  │  │  │  │  │至│
                //　└─┴─┴─┴─┴─┴─┴─┴─┴─┘

                for (int i = 0; i < 8; i++) { srcAll.Add( dst.Offset(sengo, i + 1, -(i + 1))); }
            }

            return srcAll;
        }


        public static List<MasuBango> SrcA_Oh(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│  │Ａ│  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 0, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │Ａ│  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│  │  │  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 0, -1) };
            }

            return srcAll;
        }


        public static List<MasuBango> SrcB_Oh(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│Ｂ│  │  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 1, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │  │Ｂ│
                //├─┼─┼─┤
                //│  │至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, -1, -1) };
            }

            return srcAll;
        }


        public static List<MasuBango> SrcC_Oh(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│Ｃ│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 1, 0) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│  │  │  │
                //├─┼─┼─┤
                //│　│至│Ｃ│
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, -1, 0) };
            }

            return srcAll;
        }


        public static List<MasuBango> SrcD_Oh(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│Ｄ│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 1, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │Ｄ│
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, -1, 1) };
            }

            return srcAll;
        }


        public static List<MasuBango> SrcE_Oh(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;


            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┬─┬─┐
                //│　│Ｅ│  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 0, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┬─┬─┐
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│Ｅ│  │
                //└─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 0, 1) };
            }


            return srcAll;
        }


        public static List<MasuBango> SrcF_Oh(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │Ｆ│
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, -1, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │Ｆ│  │  │
                //  └─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 1, 1) };
            }

            return srcAll;
        }


        public static List<MasuBango> SrcG_Oh(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│Ｇ│
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, -1, 0) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │Ｇ│至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 1, 0) };
            }

            return srcAll;
        }


        public static List<MasuBango> SrcH_Oh(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //  ┌─┬─┬─┐
                //  │  │  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │Ｈ│
                //  └─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, -1, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //  ┌─┬─┬─┐
                //  │Ｈ│  │  │
                //  ├─┼─┼─┤
                //  │  │至│  │
                //  ├─┼─┼─┤
                //  │  │  │  │
                //  └─┴─┴─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 1, -1) };
            }

            return srcAll;
        }


        public static List<MasuBango> SrcI_Kei(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┐　┌─┐
                //│Ｉ│　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┘　└─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 1, -2) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┐　┌─┐
                //│  │　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│  │　│Ｉ│
                //└─┘　└─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, -1, 2) };
            }

            return srcAll;
        }


        public static List<MasuBango> SrcJ_Kei(Sengo sengo, MasuBango dst)
        {
            List<MasuBango> srcAll;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┐　┌─┐
                //│  │　│Ｊ│
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│  │  │  │
                //└─┘　└─┘

                //----------
                // 競合駒マス(pre masu)
                //----------
                srcAll = new List<MasuBango>() { dst.Offset(sengo, -1, -2) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┐　┌─┐
                //│  │　│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│至│  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│Ｊ│　│  │
                //└─┘　└─┘

                srcAll = new List<MasuBango>() { dst.Offset(sengo, 1, 2) };
            }

            return srcAll;
        }

    }

}
