﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Xenon.KifuNarabe
{

    /// <summary>
    /// 局面
    /// </summary>
    public class Kyokumen
    {

        #region
        //------------------------------
        /// <summary>
        /// 駒
        /// </summary>
        public List<Button_Koma> Items
        {
            get;
            set;
        }
        //------------------------------
        #endregion


        public Kyokumen()
        {
            this.Items = new List<Button_Koma>();
        }



        /// <summary>
        /// 指定の場所にある駒を返します。無ければヌル。
        /// </summary>
        /// <param name="okiba">置き場</param>
        /// <param name="masu">筋、段</param>
        /// <param name="uc_Main">メインパネル</param>
        /// <returns>駒</returns>
        public Button_Koma AtMasuDokodemo(Okiba okiba, MasuBango masu)
        {
            Button_Koma foundKoma = null;

            foreach (Button_Koma k in this.Items)
            {
                if (k.Masu.Okiba == okiba
                    && k.Masu.Suji == masu.Suji
                    && k.Masu.Dan == masu.Dan)
                {
                    foundKoma = k;
                    break;
                }
            }

            return foundKoma;
        }

        /// <summary>
        /// 指定の場所にある駒を返します。無ければヌル。
        /// </summary>
        /// <param name="okiba">置き場</param>
        /// <param name="masu">筋、段</param>
        /// <param name="uc_Main">メインパネル</param>
        /// <returns>駒</returns>
        public Button_Koma AtMasu(Sengo sengo, MasuBango masu)
        {
            Button_Koma foundKoma = null;

            foreach (Button_Koma btnKoma in this.Items)
            {
                if (
                    btnKoma.Masu.Sengo == sengo
                    && btnKoma.Masu.Okiba == Okiba.SHOGIBAN
                    && btnKoma.Masu.Suji == masu.Suji
                    && btnKoma.Masu.Dan == masu.Dan)
                {
                    foundKoma = btnKoma;
                    break;
                }
            }

            return foundKoma;
        }

        /// <summary>
        /// 飛車の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmA_Hisya(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prA;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ
                prA = new MasuBango[8];
                for (int i = 0; i < 8; i++) { prA[i] = masu.Offset(sengo, 0, i + 1); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //                    Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｈ│A0│Ｂ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｆ│E0│Ｄ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ｅ
                prA = new MasuBango[8];
                for (int i = 0; i < 8; i++) { prA[i] = masu.Offset(sengo, 0, -(i + 1)); }
            }


            Button_Koma[] kmA = new Button_Koma[8];
            for (int i = 0; i < 8; i++) { kmA[i] = this.AtMasu(sengo, prA[i]); }

            return kmA;
        }

        
        /// <summary>
        /// 飛車の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmC_Hisya(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prC;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ

                prC = new MasuBango[8];
                for (int i = 0; i < 8; i++) { prC[i] = masu.Offset(sengo, i + 1, 0); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //                    Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｈ│A0│Ｂ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｆ│E0│Ｄ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ｅ

                prC = new MasuBango[8];
                for (int i = 0; i < 8; i++) { prC[i] = masu.Offset(sengo, -(i + 1), 0); }
            }

            Button_Koma[] kmC = new Button_Koma[8];
            for (int i = 0; i < 8; i++) { kmC[i] = this.AtMasu(sengo, prC[i]); }

            return kmC;
        }


        /// <summary>
        /// 飛車の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmE_Hisya(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prE;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ

                prE = new MasuBango[8];
                for (int i = 0; i < 8; i++) { prE[i] = masu.Offset(sengo, 0, -(i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //                    Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｈ│A0│Ｂ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｆ│E0│Ｄ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ｅ

                prE = new MasuBango[8];
                for (int i = 0; i < 8; i++) { prE[i] = masu.Offset(sengo, 0, i + 1); }
            }

            Button_Koma[] kmE = new Button_Koma[8];
            for (int i = 0; i < 8; i++) { kmE[i] = this.AtMasu(sengo, prE[i]); }

            return kmE;
        }


        /// <summary>
        /// 飛車の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmG_Hisya(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prG;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ

                prG = new MasuBango[8];
                for (int i = 0; i < 8; i++) { prG[i] = masu.Offset(sengo, -(i + 1), 0); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //                    Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｈ│A0│Ｂ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｆ│E0│Ｄ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ｅ

                prG = new MasuBango[8];
                for (int i = 0; i < 8; i++) { prG[i] = masu.Offset(sengo, i + 1, 0); }
            }

            Button_Koma[] kmG = new Button_Koma[8];
            for (int i = 0; i < 8; i++) { kmG[i] = this.AtMasu(sengo, prG[i]); }

            return kmG;
        }

        
        /// <summary>
        /// 馬の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmB_Kaku(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prB;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│D3│  │  │  │  │  │  │  │F3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D2│  │  │  │  │  │F2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D0│Ｅ│F0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｃ│  │Ｇ│  │  │  │中心が移動先、英字は移動可能元。後手なので天地逆。
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B0│Ａ│H0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B2│  │  │  │  │  │H2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B3│  │  │  │  │  │  │  │H3│
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                prB = new MasuBango[8]; for (int i = 0; i < 8; i++) { prB[i] = masu.Offset(sengo, i + 1, i + 1); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│H3│  │  │  │  │  │  │  │B3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H2│  │  │  │  │  │B2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H0│Ａ│B0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｇ│  │Ｃ│  │  │  │中心が移動先、英字は移動候補駒
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F0│Ｅ│D0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F2│  │  │  │  │  │D2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F3│  │  │  │  │  │  │  │D3│
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                prB = new MasuBango[8]; for (int i = 0; i < 8; i++) { prB[i] = masu.Offset(sengo, -(i + 1), -(i + 1)); }
            }

            Button_Koma[] kmB = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmB[i] = this.AtMasu(sengo, prB[i]); }

            return kmB;
        }


        /// <summary>
        /// 馬の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmD_Kaku(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prD;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│D3│  │  │  │  │  │  │  │F3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D2│  │  │  │  │  │F2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D0│Ｅ│F0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｃ│  │Ｇ│  │  │  │中心が移動先、英字は移動可能元。後手なので天地逆。
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B0│Ａ│H0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B2│  │  │  │  │  │H2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B3│  │  │  │  │  │  │  │H3│
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                prD = new MasuBango[8]; for (int i = 0; i < 8; i++) { prD[i] = masu.Offset(sengo, i + 1, -(i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│H3│  │  │  │  │  │  │  │B3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H2│  │  │  │  │  │B2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H0│Ａ│B0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｇ│  │Ｃ│  │  │  │中心が移動先、英字は移動候補駒
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F0│Ｅ│D0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F2│  │  │  │  │  │D2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F3│  │  │  │  │  │  │  │D3│
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                prD = new MasuBango[8]; for (int i = 0; i < 8; i++) { prD[i] = masu.Offset(sengo, -(i + 1), i + 1); }
            }

            Button_Koma[] kmD = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmD[i] = this.AtMasu(sengo, prD[i]); }

            return kmD;
        }


        /// <summary>
        /// 馬の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmF_Kaku(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prF;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│D3│  │  │  │  │  │  │  │F3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D2│  │  │  │  │  │F2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D0│Ｅ│F0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｃ│  │Ｇ│  │  │  │中心が移動先、英字は移動可能元。後手なので天地逆。
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B0│Ａ│H0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B2│  │  │  │  │  │H2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B3│  │  │  │  │  │  │  │H3│
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                prF = new MasuBango[8]; for (int i = 0; i < 8; i++) { prF[i] = masu.Offset(sengo, -(i + 1), -(i + 1)); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│H3│  │  │  │  │  │  │  │B3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H2│  │  │  │  │  │B2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H0│Ａ│B0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｇ│  │Ｃ│  │  │  │中心が移動先、英字は移動候補駒
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F0│Ｅ│D0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F2│  │  │  │  │  │D2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F3│  │  │  │  │  │  │  │D3│
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                prF = new MasuBango[8]; for (int i = 0; i < 8; i++) { prF[i] = masu.Offset(sengo, i + 1, i + 1); }
            }

            Button_Koma[] kmF = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmF[i] = this.AtMasu(sengo, prF[i]); }

            return kmF;
        }

        /// <summary>
        /// 馬の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmH_Kaku(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prH;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│D3│  │  │  │  │  │  │  │F3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D2│  │  │  │  │  │F2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D0│Ｅ│F0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｃ│  │Ｇ│  │  │  │中心が移動先、英字は移動可能元。後手なので天地逆。
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B0│Ａ│H0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B2│  │  │  │  │  │H2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B3│  │  │  │  │  │  │  │H3│
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                prH = new MasuBango[8]; for (int i = 0; i < 8; i++) { prH[i] = masu.Offset(sengo, -(i + 1), i + 1); }
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│H3│  │  │  │  │  │  │  │B3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H2│  │  │  │  │  │B2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H0│Ａ│B0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｇ│  │Ｃ│  │  │  │中心が移動先、英字は移動候補駒
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F0│Ｅ│D0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F2│  │  │  │  │  │D2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F3│  │  │  │  │  │  │  │D3│
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                prH = new MasuBango[8]; for (int i = 0; i < 8; i++) { prH[i] = masu.Offset(sengo, i + 1, -(i + 1)); }
            }

            Button_Koma[] kmH = new Button_Koma[8]; for (int i = 0; i < 8; i++) { kmH[i] = this.AtMasu(sengo, prH[i]); }

            return kmH;
        }



        /// <summary>
        /// 王の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmA_Oh(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prA;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│D3│  │  │  │  │  │  │  │F3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D2│  │  │  │  │  │F2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D0│Ｅ│F0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｃ│  │Ｇ│  │  │  │中心が移動先、英字は移動可能元。後手なので天地逆。
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B0│Ａ│H0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B2│  │  │  │  │  │H2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B3│  │  │  │  │  │  │  │H3│
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                prA = new MasuBango[] { masu.Offset(sengo, 0, 1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│H3│  │  │  │  │  │  │  │B3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H2│  │  │  │  │  │B2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H0│Ａ│B0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｇ│  │Ｃ│  │  │  │中心が移動先、英字は移動候補駒
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F0│Ｅ│D0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F2│  │  │  │  │  │D2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F3│  │  │  │  │  │  │  │D3│
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                prA = new MasuBango[] { masu.Offset(sengo, 0, -1) };
            }

            Button_Koma[] kmA = new Button_Koma[] { this.AtMasu(sengo, prA[0]) };

            return kmA;
        }



        /// <summary>
        /// 王の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmB_Oh(Sengo sengo, MasuBango masu)
        {
            MasuBango prB;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ

                prB = masu.Offset(sengo, 1, 1);
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //                    Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｈ│A0│Ｂ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｆ│E0│Ｄ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ｅ

                prB = masu.Offset(sengo, -1, -1);
            }

            Button_Koma[] kmB = new Button_Koma[] { this.AtMasu(sengo, prB) };

            return kmB;
        }


        /// <summary>
        /// 王の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmC_Oh(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prC;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│D3│  │  │  │  │  │  │  │F3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D2│  │  │  │  │  │F2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D0│Ｅ│F0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｃ│  │Ｇ│  │  │  │中心が移動先、英字は移動可能元。後手なので天地逆。
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B0│Ａ│H0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B2│  │  │  │  │  │H2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B3│  │  │  │  │  │  │  │H3│
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                prC = new MasuBango[] { masu.Offset(sengo, 1, 0) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│H3│  │  │  │  │  │  │  │B3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H2│  │  │  │  │  │B2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H0│Ａ│B0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｇ│  │Ｃ│  │  │  │中心が移動先、英字は移動候補駒
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F0│Ｅ│D0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F2│  │  │  │  │  │D2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F3│  │  │  │  │  │  │  │D3│
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                prC = new MasuBango[] { masu.Offset(sengo, -1, 0) };
            }

            Button_Koma[] kmC = new Button_Koma[] { this.AtMasu(sengo, prC[0]) };

            return kmC;
        }


        /// <summary>
        /// 王の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmD_Oh(Sengo sengo, MasuBango masu)
        {
            MasuBango prD;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ

                prD = masu.Offset(sengo, 1, -1);
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //                    Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｈ│A0│Ｂ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｆ│E0│Ｄ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ｅ

                prD = masu.Offset(sengo, -1, 1);
            }

            Button_Koma[] kmD = new Button_Koma[] { this.AtMasu(sengo, prD) };

            return kmD;
        }


        /// <summary>
        /// 王の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmE_Oh(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prE;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //Ｄ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｆ
                //　│D3│  │  │  │  │  │  │  │F3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │D2│  │  │  │  │  │F2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │D1│  │  │  │F1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │D0│Ｅ│F0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｃ│  │Ｇ│  │  │  │中心が移動先、英字は移動可能元。後手なので天地逆。
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │B0│Ａ│H0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │B1│  │  │  │H1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │B2│  │  │  │  │  │H2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│B3│  │  │  │  │  │  │  │H3│
                //Ｂ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｈ

                prE = new MasuBango[] { masu.Offset(sengo, 0, -1) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│H3│  │  │  │  │  │  │  │B3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H2│  │  │  │  │  │B2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H0│Ａ│B0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｇ│  │Ｃ│  │  │  │中心が移動先、英字は移動候補駒
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F0│Ｅ│D0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F2│  │  │  │  │  │D2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F3│  │  │  │  │  │  │  │D3│
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                prE = new MasuBango[] { masu.Offset(sengo, 0, 1) };
            }

            Button_Koma[] kmE = new Button_Koma[] { this.AtMasu(sengo, prE[0]) };

            return kmE;
        }


        /// <summary>
        /// 王の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmF_Oh(Sengo sengo, MasuBango masu)
        {
            MasuBango prF;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ

                prF = masu.Offset(sengo, -1, -1);
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //                    Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｈ│A0│Ｂ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｆ│E0│Ｄ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ｅ

                prF = masu.Offset(sengo, 1, 1);
            }

            Button_Koma[] kmF = new Button_Koma[] { this.AtMasu(sengo, prF) };

            return kmF;
        }


                /// <summary>
        /// 王の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmG_Oh(Sengo sengo, MasuBango masu)
        {
            MasuBango[] prG;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ

                prG = new MasuBango[] { masu.Offset(sengo, -1, 0) };
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //Ｈ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐Ｂ
                //　│H3│  │  │  │  │  │  │  │B3│
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │H2│  │  │  │  │  │B2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │H1│  │  │  │B1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │H0│Ａ│B0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │Ｇ│  │Ｃ│  │  │  │中心が移動先、英字は移動候補駒
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │  │F0│Ｅ│D0│  │  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │  │F1│  │  │  │D1│  │  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│  │F2│  │  │  │  │  │D2│  │
                //　├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //　│F3│  │  │  │  │  │  │  │D3│
                //Ｆ└─┴─┴─┴─┴─┴─┴─┴─┴─┘Ｄ

                prG = new MasuBango[] { masu.Offset(sengo, 1, 0) };
            }

            Button_Koma[] kmG = new Button_Koma[] { this.AtMasu(sengo, prG[0]) };

            return kmG;
        }

        
        /// <summary>
        /// 王の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmH_Oh(Sengo sengo, MasuBango masu)
        {
            MasuBango prH;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //                    Ｅ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │E3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｄ│E0│Ｆ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｃ│C3│C2│C1│C0│  │G0│G1│G2│G3│Ｇ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｂ│A0│Ｈ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動可能元。後手なので天地逆。
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ａ

                prH = masu.Offset(sengo, -1, 1);
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //                    Ａ
                //  ┌─┬─┬─┬─┬─┬─┬─┬─┬─┐
                //  │  │  │  │  │A3│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │A1│  │  │  │  │　中心が移動先、英字は移動候補駒
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｈ│A0│Ｂ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //Ｇ│G3│G2│G1│G0│  │C0│C1│C2│C3│Ｃ
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │Ｆ│E0│Ｄ│  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E1│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E2│  │  │  │  │
                //  ├─┼─┼─┼─┼─┼─┼─┼─┼─┤
                //  │  │  │  │  │E3│  │  │  │  │
                //  └─┴─┴─┴─┴─┴─┴─┴─┴─┘
                //                    Ｅ

                prH = masu.Offset(sengo, 1, -1);
            }

            Button_Koma[] kmH = new Button_Koma[] { this.AtMasu(sengo, prH) };

            return kmH;
        }


        /// <summary>
        /// 桂の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmI_Kei(Sengo sengo, MasuBango masu)
        {
            MasuBango prI;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┐　┌─┐
                //│Ｉ│　│Ｊ│中心が移動先、英字は移動候補駒。後手なので天地逆。
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┘　└─┘

                prI = masu.Offset(sengo, 1, -2);
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┐　┌─┐
                //│  │　│  │中心が移動先、英字は移動候補駒
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│Ｊ│　│Ｉ│
                //└─┘　└─┘

                prI = masu.Offset(sengo, -1, 2);
            }

            Button_Koma[] kmI = new Button_Koma[] { this.AtMasu(sengo, prI) };

            return kmI;
        }


        /// <summary>
        /// 桂の軌道上の駒
        /// </summary>
        /// <param name="km"></param>
        /// <returns></returns>
        public Button_Koma[] KmJ_Kei(Sengo sengo, MasuBango masu)
        {
            MasuBango prJ;

            if (Sengo.GOTE == sengo)
            {
                //******************************
                // △後手
                //******************************
                //┌─┐　┌─┐
                //│Ｉ│　│Ｊ│中心が移動先、英字は移動候補駒。後手なので天地逆。
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //└─┘　└─┘

                prJ = masu.Offset(sengo, -1, -2);
            }
            else
            {
                //******************************
                // ▲先手
                //******************************
                //┌─┐　┌─┐
                //│  │　│  │中心が移動先、英字は移動候補駒
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│　│  │  │
                //├─┼─┼─┤
                //│Ｊ│　│Ｉ│
                //└─┘　└─┘

                prJ = masu.Offset(sengo, 1, 2);
            }

            Button_Koma[] kmJ = new Button_Koma[] { this.AtMasu(sengo, prJ) };

            return kmJ;
        }

    }
}
